# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import io
import bson
from contextlib import redirect_stdout
from superdesk.tests import TestCase
from superdesk import get_resource_service
from superdesk.commands.delete_archived_document import DeleteArchivedDocumentCommand


class DeleteDocTestCase(TestCase):
    def setUp(self):
        self.guid = "urn:newsml:localhost:2016-09-12T12:11:40.160498:7237e59f-c42d-4865-aee5-e364aeb2966a"

        self.archived_only_data = [
            {
                "_id": bson.ObjectId("588c1b901d41c805dce70df0"),
                "guid": self.guid,
                "item_id": self.guid,
                "_current_version": 3,
                "type": "text",
                "abstract": "test",
                "state": "fetched",
                "slugline": "slugline",
                "headline": "headline",
                "flags": {"marked_archived_only": True},
                "subject": [{"qcode": "17004000", "name": "Statistics"}],
                "body_html": "Test Document body",
                "source": "AAP",
                "linked_in_packages": [
                    {
                        "package": "urn:newsml:localhost:2017-01-28T15:28:33.535974:30c01757-cfb5-4985-a8f4-c3ecb253a244",
                        "package_type": "takes",
                    }
                ],
            },
            {
                "_id": bson.ObjectId("57d224de069b7f038e9d2a53"),
                "guid": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
                "item_id": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
                "_current_version": 4,
                "type": "text",
                "abstract": "test",
                "state": "fetched",
                "slugline": "slugline",
                "headline": "headline",
                "flags": {"marked_archived_only": True},
                "subject": [{"qcode": "17004000", "name": "Statistics"}],
                "body_html": "Test Document body",
                "source": "AAP",
            },
            {
                "_id": "213456",
                "guid": self.guid,
                "item_id": "urn:newsml:localhost:2017-01-28T15:28:33.535974:30c01757-cfb5-4985-a8f4-c3ecb253a244",
                "_current_version": 1,
                "type": "composite",
                "abstract": "test",
                "state": "fetched",
                "slugline": "slugline",
                "headline": "headline",
                "groups": [
                    {"id": "root", "refs": [{"idRef": "main"}], "role": "grpRole:NEP"},
                    {
                        "id": "main",
                        "refs": [
                            {
                                "residRef": "urn:newsml:localhost:2016-09-12T12:11:40.160498:7237e59f-c42d-4865-aee5-e364aeb2966a",
                                "guid": "urn:newsml:localhost:2016-09-12T12:11:40.160498:7237e59f-c42d-4865-aee5-e364aeb2966a",
                                "itemClass": "icls:text",
                                "headline": "Testing takes",
                                "type": "text",
                                "renditions": {},
                                "_current_version": 3,
                                "is_published": True,
                                "slugline": "Takes test",
                                "sequence": 1,
                                "location": "archived",
                            },
                            {
                                "residRef": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
                                "guid": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
                                "itemClass": "icls:text",
                                "headline": "Testing takes",
                                "type": "text",
                                "renditions": {},
                                "_current_version": 4,
                                "is_published": True,
                                "slugline": "Takes test",
                                "sequence": 2,
                                "location": "archived",
                            },
                        ],
                        "role": "grpRole:main",
                    },
                ],
            },
        ]

        self.archivedService = get_resource_service("archived")

    def test_no_id_provided_exception(self):
        self.archivedService.post(self.archived_only_data)

        f = io.StringIO()
        with redirect_stdout(f):
            DeleteArchivedDocumentCommand().run([])
        s = f.getvalue()
        self.assertEqual(s, "Please provide at least one id!\n")

    def test_wrong_id_provided_exception(self):
        self.archivedService.post(self.archived_only_data)

        f = io.StringIO()
        with redirect_stdout(f):
            DeleteArchivedDocumentCommand().run(["5880000000000"])
        s = f.getvalue()
        self.assertIn("No archived story found with given ids(s)!\n", s)

    def test_delete_non_text_document_succeeds(self):
        self.archivedService.post(self.archived_only_data)

        DeleteArchivedDocumentCommand().run(["213456"])
        cursor = self.archivedService.get(req=None, lookup={"_id": "213456"})
        self.assertEqual(0, len(cursor.docs))

    def test_delete_document_succeeds(self):
        self.archivedService.post(self.archived_only_data)
        DeleteArchivedDocumentCommand().run(["588c1b901d41c805dce70df0"])

        cursor = self.archivedService.get(req=None, lookup={"_id": "588c1b901d41c805dce70df0"})
        self.assertEqual(0, len(cursor.docs))
        cursor = self.archivedService.get(req=None, lookup={"_id": "213456"})
        self.assertEqual(0, len(cursor.docs))

    def test_delete_multiple_documents_succeeds(self):
        self.archivedService.post(self.archived_only_data)
        DeleteArchivedDocumentCommand().run(["588c1b901d41c805dce70df0", "57d224de069b7f038e9d2a53"])

        cursor = self.archivedService.get(req=None, lookup={"_id": "588c1b901d41c805dce70df0"})
        self.assertEqual(0, len(cursor.docs))
        cursor = self.archivedService.get(req=None, lookup={"_id": "57d224de069b7f038e9d2a53"})
        self.assertEqual(0, len(cursor.docs))

    def test_deleting_one_take_deletes_package_but_keeps_other_takes_succeeds(self):
        # it will delete other takes in that package
        self.archivedService.post(self.archived_only_data)
        DeleteArchivedDocumentCommand().run(["588c1b901d41c805dce70df0"])

        cursor = self.archivedService.get(req=None, lookup={"_id": "588c1b901d41c805dce70df0"})
        self.assertEqual(0, len(cursor.docs))
        cursor = self.archivedService.get(req=None, lookup={"_id": "213456"})
        self.assertEqual(0, len(cursor.docs))
        cursor = self.archivedService.get(req=None, lookup={"_id": "57d224de069b7f038e9d2a53"})
        self.assertEqual(1, len(cursor.docs))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
from uuid import uuid4

import superdesk
from werkzeug.datastructures import ImmutableMultiDict
from eve.utils import ParsedRequest
from superdesk.text_utils import get_text
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from apps.auth import get_user


class ConceptItemsService(BaseService):
    """
    CRUD service for concept items
    """

    def get(self, req, lookup):
        if all([req, req.sort]):
            CASE_INSENSITIVE_COLLATION = '{"locale": "en", "strength":"1"}'
            COLLATION_SORT_ARGS = (
                "name",
                "-name",
                "definition_html",
                "-definition_html",
                "definition_text",
                "-definition_text",
            )

            # apply case insensitive collation only if collation was not provided explicitly
            # and sorting by `name`, `definition_html` or `definition_text` was used
            req_sort_args = [arg.strip() for arg in req.sort.split(",")]
            if [arg for arg in req_sort_args if arg in COLLATION_SORT_ARGS] and (
                not req.args or not req.args.get("collation")
            ):
                if req.args:
                    req.args = req.args.to_dict()
                    req.args["collation"] = CASE_INSENSITIVE_COLLATION
                    req.args = ImmutableMultiDict(req.args)
                else:
                    req.args = ImmutableMultiDict({"collation": CASE_INSENSITIVE_COLLATION})

            # if sort by (-)definition_html was used, we'll apply sort by definition_text under the hood
            if any([arg for arg in req_sort_args if arg in ("definition_html", "-definition_html")]):
                req_sort_args = ["definition_text" if arg == "definition_html" else arg for arg in req_sort_args]
                req_sort_args = ["-definition_text" if arg == "-definition_html" else arg for arg in req_sort_args]
                if req.args:
                    req.args = req.args.to_dict()
                    req.args["sort"] = ",".join(set(req_sort_args))
                    req.args = ImmutableMultiDict(req.args)
                else:
                    req.args = ImmutableMultiDict({"sort": ",".join(set(req_sort_args))})
                req.sort = req.args["sort"]

        return super().get(req, lookup)

    def on_create(self, docs):
        for doc in docs:
            self._validate_properties(doc)
            self._validate_language(doc)
            self._setup_created_by(doc)
            self._setup_group_id(doc)
            self._fill_definition_text(doc)

    def on_replace(self, doc, original):
        self._validate_properties(doc)
        self._validate_language(doc)
        self._validate_group_id(doc)
        self._setup_created_by(doc, original)
        self._setup_updated_by(doc)
        self._fill_definition_text(doc)

    def on_update(self, updates, original):
        if "cpnat_type" in updates:
            # we must validate `properties` even if they were not in payload
            if "properties" not in updates and "properties" in original:
                updates["properties"] = original["properties"]
        elif "properties" in updates:
            # we must validate `cpnat_type` even if it was not in payload
            updates["cpnat_type"] = original["cpnat_type"]
        if "cpnat_type" in updates:
            self._validate_properties(updates)

        if "language" in updates:
            self._validate_language(updates)

        if "definition_html" in updates:
            self._fill_definition_text(updates)

        self._setup_updated_by(updates)

    def _validate_language(self, doc):
        # fetch languages from CVs
        req = ParsedRequest()
        req.projection = json.dumps({"items.qcode": 1})

        try:
            languages = (
                superdesk.get_resource_service("vocabularies").find_one(req=req, _id="languages").get("items", [])
            )
        except AttributeError:
            raise SuperdeskApiError.badRequestError(
                message="Request is not valid",
                payload={"language": "Concept items requires 'languages' vocabulary to be set"},
            )

        languages_qcodes = [lang["qcode"] for lang in languages]

        if doc["language"] not in languages_qcodes:
            raise SuperdeskApiError.badRequestError(
                message="Request is not valid", payload={"language": "unallowed value '{}'".format(doc["language"])}
            )

    def _validate_group_id(self, doc):
        if "group_id" not in doc:
            raise SuperdeskApiError.badRequestError(
                message="Request is not valid", payload={"group_id": "This field is required"}
            )

    def _fill_definition_text(self, doc):
        doc["definition_text"] = get_text(doc["definition_html"], content="html", lf_on_block=True).strip()

    def _validate_properties(self, doc):
        getattr(self, "_validate_{}_properties".format(doc["cpnat_type"].rsplit(":", 1)[-1].lower()))(doc)

    def _validate_abstract_properties(self, doc):
        if "properties" in doc:
            raise SuperdeskApiError.badRequestError(
                message="Request is not valid",
                payload={"properties": "field is not supported when 'cpnat_type' is 'cpnat:abstract'"},
            )

    def _validate_event_properties(self, doc):
        # Step to implement validation(s):
        # 1) define schema_event in resource (https://iptc.org/std/NewsML-G2/guidelines/#more-real-world-entities)
        # 2) create cerberus Validator based on the schema_event
        # 3) validate doc['properties'] using cerberus Validator instance
        # TODO if we want to have a case insensitive sort, please, add a proper index

        raise SuperdeskApiError.badRequestError(
            message="Request is not valid", payload={"cpnat_type": "concept type 'cpnat:event' is not supported"}
        )

    def _validate_geoarea_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid", payload={"cpnat_type": "concept type 'cpnat:geoArea' is not supported"}
        )

    def _validate_object_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid", payload={"cpnat_type": "concept type 'cpnat:object' is not supported"}
        )

    def _validate_organisation_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid", payload={"cpnat_type": "concept type 'cpnat:organisation' is not supported"}
        )

    def _validate_person_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid", payload={"cpnat_type": "concept type 'cpnat:person' is not supported"}
        )

    def _validate_poi_properties(self, doc):
        raise SuperdeskApiError.badRequestError(
            message="Request is not valid", payload={"cpnat_type": "concept type 'cpnat:poi' is not supported"}
        )

    def _setup_created_by(self, doc, original=None):
        if not original:
            doc["created_by"] = get_user().get("_id")
        else:
            # for PUT request
            doc["created_by"] = original["created_by"]

    def _setup_updated_by(self, doc):
        doc["updated_by"] = get_user().get("_id")

    def _setup_group_id(self, doc):
        if "group_id" not in doc:
            doc["group_id"] = str(uuid4())



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : petr
# Creation: 2019-09-24 10:49

from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "roles"
    privileges = [
        "publisher_dashboard",
        "planning_assignments_view",
        "monitoring_view",
        "spike_read",
        "highlights_read",
        "use_global_saved_searches",
        "dashboard",
        "ansa_metasearch",
        "ansa_live_assistant",
        "ansa_ai_news",
    ]

    def forwards(self, mongodb_collection, mongodb_database):
        updates = {}
        for privilege in self.privileges:
            updates["privileges.{}".format(privilege)] = 1

        result = mongodb_collection.update_many({}, {"$set": updates})
        print("updated {}/{} roles".format(result.modified_count, result.matched_count))

    def backwards(self, mongodb_collection, mongodb_database):
        pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import render_template_string

from superdesk.tests import TestCase


class ConvertDatetimeFiltersTest(TestCase):
    def test_convert_datetime_utc_no_format(self):
        template_string = '{{ item.versioncreated | format_datetime("Australia/Sydney")}}'
        item = {"versioncreated": "2015-01-01T22:54:53+0000"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "2015-01-02 09:54:53+11:00")

    def test_convert_datetime_local_time_no_format(self):
        template_string = '{{ item.versioncreated | format_datetime("Australia/Sydney")}}'
        item = {"versioncreated": "2015-01-01T22:54:53+05:30"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "2015-01-02 04:24:53+11:00")

    def test_convert_datetime_utc_format(self):
        template_string = (
            '{{ item.versioncreated | format_datetime(timezone_string="Australia/Sydney", ' 'date_format="%Y-%m-%d")}}'
        )
        item = {"versioncreated": "2015-01-01T22:54:53+0000"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "2015-01-02")

    def test_convert_datetime_invalid_date(self):
        template_string = '{{ item.versioncreated | format_datetime("Australia/Sydney", "%Y-%m-%d")}}'
        item = {"versioncreated": "test string"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "")

    def test_convert_datetime_invalid_timezone(self):
        template_string = '{{ item.versioncreated | format_datetime("australia/sydney", "%Y-%m-%d")}}'
        item = {"versioncreated": "test string"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "")

    def test_convert_datetime_utc_default_timezone(self):
        template_string = "{{ item.versioncreated | format_datetime()}}"
        item = {"versioncreated": "2015-01-01T22:54:53+0000"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "2015-01-01 23:54:53+01:00")

    def test_convert_datetime_local_time_default_timezone(self):
        template_string = "{{ item.versioncreated | format_datetime()}}"
        item = {"versioncreated": "2015-01-01T22:54:53+05:30"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "2015-01-01 18:24:53+01:00")

    def test_convert_datetime_utc_timezone_format(self):
        template_string = '{{ item.versioncreated | format_datetime("Australia/Sydney", "%d %b %Y %H:%S %Z")}}'
        item = {"versioncreated": "2015-01-01T22:54:53+0000"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "02 Jan 2015 09:53 AEDT")

        item = {"versioncreated": "2015-06-01T22:54:53+0000"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "02 Jun 2015 08:53 AEST")

    def test_get_first_paragraph(self):
        template_string = "{{ item.body_html | first_paragraph() }}"
        item = {"body_html": "<p><br></p><p>First paragraph</p><p>Second paragraph</p>"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "<p>First paragraph</p>")

    def test_get_first_paragraph_doesnt_fail_with_empty_body(self):
        template_string = "{{ item.body_html | first_paragraph()}}"
        item = {"headline": "Sample headline"}
        result = render_template_string(template_string, item=item)
        self.assertEqual(result, "")



from superdesk.text_utils import get_text
from .ninjs_formatter import NINJSFormatter


def format_datetime(date):
    return date.isoformat()


class IMatricsFormatter(NINJSFormatter):
    def can_format(self, format_type, article):
        return format_type.lower() == "imatrics" and article.get("type") == "text"

    def _transform_to_ninjs(self, article, subscriber, recursive=True):
        return {
            "uuid": article["guid"],
            "createdTimestamp": format_datetime(article["firstcreated"]),
            "latestVersionTimestamp": format_datetime(article["versioncreated"]),
            "publicationTimestamp": format_datetime(article["firstpublished"]),
            "authors": [author["sub_label"] for author in article.get("authors") or []],
            "language": article["language"],
            "pubStatus": True,
            "concepts": self._format_concepts(article),
            "headline": get_text(article["headline"]),
            "preamble": get_text(article["abstract"], lf_on_block=True).strip() if article.get("abstract") else "",
            "dateline": article["dateline"]["text"]
            if article.get("dateline") and article["dateline"].get("text")
            else "",
            "body": [line.strip() for line in get_text(article["body_html"], lf_on_block=True).split("\n") if line],
        }

    def _format_concepts(self, article):
        concepts = []
        if article.get("subject"):
            concepts.extend(
                [
                    {
                        "type": "topic" if subj.get("scheme") == "imatrics_topic" else "category",
                        "title": subj["name"],
                        "uuid": subj["altids"]["imatrics"],
                    }
                    for subj in article["subject"]
                    if subj.get("altids") and subj["altids"].get("imatrics")
                ]
            )
        for _type in ("organisation", "person", "place", "event", "object"):
            if not article.get(_type):
                continue
            concepts.extend(
                [
                    {
                        "type": _type,
                        "title": concept["name"],
                        "uuid": concept["altids"]["imatrics"],
                    }
                    for concept in article[_type]
                    if concept.get("altids") and concept["altids"].get("imatrics")
                ]
            )
        return concepts



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock
from nose.tools import assert_raises

from superdesk.tests import TestCase
from superdesk.media.crop import CropService
from superdesk.errors import SuperdeskApiError
from superdesk.media.media_operations import crop_image
from superdesk.media.renditions import _resize_image, get_renditions_spec, can_generate_custom_crop_from_original
from apps.prepopulate.app_populate import populate_table_json

from ..media import get_picture_fixture


class CropTestCase(TestCase):

    crop_sizes = {
        "_id": "crop_sizes",
        "display_name": "Image Crop Sizes",
        "type": "manageable",
        "items": [
            {"is_active": True, "name": "4-3", "width": 800, "height": 600},
            {"is_active": True, "name": "16-9", "width": 1280, "height": 720},
        ],
    }

    def setUp(self):
        self.service = CropService()
        populate_table_json("vocabularies", [self.crop_sizes])

    def test_validate_aspect_ratio_fails(self):
        doc = {"CropLeft": 0, "CropRight": 80, "CropTop": 0, "CropBottom": 60}
        crop = {"height": 700, "width": 70}
        with assert_raises(SuperdeskApiError):
            self.service._validate_aspect_ratio(crop, doc)

    def test_validate_aspect_ratio_fails_with_cropsize_less(self):
        doc = {"CropLeft": 0, "CropRight": 80, "CropTop": 0, "CropBottom": 60}
        crop = {"height": 600, "width": 800}
        with assert_raises(SuperdeskApiError):
            self.service._validate_aspect_ratio(crop, doc)

    def test_validate_aspect_ratio_succeeds(self):
        doc = {"CropLeft": 0, "CropRight": 800, "CropTop": 0, "CropBottom": 600}
        crop = {"height": 600, "width": 800}
        self.assertIsNone(self.service._validate_aspect_ratio(crop, doc))

    def test_validate_aspect_ratio_succeeds_2(self):
        doc = {"CropLeft": 0, "CropRight": 1600, "CropTop": 0, "CropBottom": 1200}
        crop = {"height": 600, "width": 800}
        self.assertIsNone(self.service._validate_aspect_ratio(crop, doc))

    def test_get_crop_by_name(self):
        self.assertIsNotNone(self.service.get_crop_by_name("16-9"))
        self.assertIsNotNone(self.service.get_crop_by_name("4-3"))
        self.assertIsNone(self.service.get_crop_by_name("d"))

    def test_validate_crop_raises_error_if_item_is_not_picture(self):
        original = {"type": "text"}
        doc = {"renditions": {"4-3": {"CropLeft": 0, "CropRight": 800, "CropTop": 0, "CropBottom": 600}}}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.validate_crop(original, doc, "4-3")

        ex = context.exception
        self.assertEqual(ex.message, "Only images can be cropped!")
        self.assertEqual(ex.status_code, 400)

    def test_validate_crop_raises_error_if_renditions_are_missing(self):
        original = {"type": "picture"}
        doc = {"renditions": {"4-3": {"CropLeft": 0, "CropRight": 800, "CropTop": 0, "CropBottom": 600}}}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.validate_crop(original, doc, "4-3")

        ex = context.exception
        self.assertEqual(ex.message, "Missing renditions!")
        self.assertEqual(ex.status_code, 400)

    def test_validate_crop_raises_error_if_original_rendition_is_missing(self):
        original = {
            "type": "picture",
            "renditions": {"4-3": {"CropLeft": 0, "CropRight": 800, "CropTop": 0, "CropBottom": 600}},
        }
        doc = {"renditions": {"4-3": {"CropLeft": 0, "CropRight": 800, "CropTop": 0, "CropBottom": 600}}}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.validate_crop(original, doc, "4-3")

        ex = context.exception
        self.assertEqual(ex.message, "Missing original rendition!")
        self.assertEqual(ex.status_code, 400)

    def test_validate_crop_raises_error_if_crop_name_is_unknown(self):
        original = {
            "type": "picture",
            "renditions": {"original": {"CropLeft": 0, "CropRight": 800, "CropTop": 0, "CropBottom": 600}},
        }
        doc = {"renditions": {"d": {"CropLeft": 0, "CropRight": 800, "CropTop": 0, "CropBottom": 600}}}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.validate_crop(original, doc, "d")

        ex = context.exception
        self.assertEqual(ex.message, "Unknown crop name! (name=d)")
        self.assertEqual(ex.status_code, 400)

    def test_add_crop_raises_error_if_original_missing(self):
        original = {"renditions": {"4-3": {}}}
        doc = {"CropLeft": 0, "CropRight": 800, "CropTop": 0, "CropBottom": 600}
        with self.assertRaises(SuperdeskApiError) as context:
            self.service.create_crop(original, "4-3", doc)

        ex = context.exception
        self.assertEqual(ex.message, "Original file couldn't be found")
        self.assertEqual(ex.status_code, 400)

    def test_validate_crop_converts_to_int(self):
        crop = {"width": "300", "height": 200}
        self.service._validate_values(crop)
        self.assertEqual(300, crop["width"])
        self.assertEqual(200, crop["height"])

        with self.assertRaises(SuperdeskApiError) as context:
            self.service._validate_values({"width": "foo"})
            self.assertEqual(context.exception.message, "Invalid value for width in renditions")

    @mock.patch("superdesk.media.crop.crop_image", return_value=(False, "test"))
    def test_add_crop_raises_error(self, crop_name):
        original = {"renditions": {"original": {}}}

        media = mock.MagicMock()
        media.name = "test.jpg"

        with mock.patch.object(self.app.media, "fetch_rendition", return_value=media):
            doc = {"CropLeft": 0, "CropRight": 800, "CropTop": 0, "CropBottom": 600}
            with self.assertRaises(SuperdeskApiError) as context:
                self.service.create_crop(original, "4-3", doc)

            ex = context.exception
            self.assertEqual(ex.message, "Saving crop failed.")
            self.assertEqual(ex.status_code, 400)

    def test_crop_image_exact_size(self):
        img = get_picture_fixture()
        size = {"width": "300", "height": "200"}
        crop = {"CropTop": "0", "CropRight": "300", "CropBottom": "200", "CropLeft": "0"}
        with open(img, "rb") as imgfile:
            res = crop_image(imgfile, img, crop, size)
            self.assertTrue(res[0])
            self.assertEqual(300, res[1].width)
            self.assertEqual(200, res[1].height)

    def test_resize_image(self):
        img = get_picture_fixture()
        with open(img, "rb") as imgfile:
            resized, width, height = _resize_image(imgfile, ("200", None), "jpeg")
            self.assertEqual(150, height)

    def test_get_rendition_spec_no_custom_crop(self):
        renditions = get_renditions_spec(no_custom_crops=True)
        for crop in self.crop_sizes.get("items"):
            self.assertNotIn(crop["name"], renditions)

    def test_get_rendition_spec_with_custom_crop(self):
        renditions = get_renditions_spec()
        for crop in self.crop_sizes.get("items"):
            self.assertIn(crop["name"], renditions)

    def test_can_generate_custom_crop_from_original(self):
        self.assertEquals(True, can_generate_custom_crop_from_original(800, 600, {"ratio": "16:9"}))
        self.assertEquals(True, can_generate_custom_crop_from_original(800, 600, {"width": 800, "height": 600}))
        self.assertEquals(True, can_generate_custom_crop_from_original(810, 600, {"width": 800, "height": 600}))
        self.assertEquals(True, can_generate_custom_crop_from_original(810, 610, {"width": 800, "height": 600}))
        self.assertEquals(False, can_generate_custom_crop_from_original(780, 610, {"width": 800, "height": 600}))
        self.assertEquals(False, can_generate_custom_crop_from_original(780, 590, {"width": 800, "height": 600}))
        self.assertEquals(True, can_generate_custom_crop_from_original(780, 590, {"width": 800}))
        self.assertEquals(True, can_generate_custom_crop_from_original(780, 590, {"height": 800}))
        self.assertEquals(False, can_generate_custom_crop_from_original(780, 590, None))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase
from superdesk import get_resource_service


class RolesTestCase(TestCase):

    roles = [{"name": "test", "privileges": {"ingest": 1, "archive": 1, "fetch": 1}}]

    users = [
        {
            "username": "foobar",
            "first_name": "foo",
            "last_name": "bar",
            "user_type": "user",
            "display_name": "Foo Bar",
            "is_enabled": True,
            "is_active": True,
        }
    ]

    def setUp(self):
        self.app.data.insert("roles", self.roles)
        self.users[0]["role"] = self.roles[0]["_id"]
        self.app.data.insert("users", self.users)

    def test_invoking_on_revoked_privileges_event(self):
        def on_revoke_roles(role, role_users):
            self.assertEqual(role.get("name"), "test")
            self.assertEqual(len(role_users), 1)
            self.assertEqual(role_users[0]["username"], "foobar")
            pass

        self.app.on_role_privileges_updated -= on_revoke_roles
        self.app.on_role_privileges_updated += on_revoke_roles
        get_resource_service("roles").patch(self.roles[0]["_id"], {"privileges": {"ingest": 0}})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from bson import ObjectId
from unittest import mock
from unittest.mock import MagicMock, ANY
from datetime import timedelta

from superdesk.tests import TestCase
from superdesk.utc import utcnow
from superdesk.errors import PublishHTTPPushServerError, PublishHTTPPushClientError
from superdesk.publish.publish_content import transmit_item


class TransmitItemTestCase(TestCase):
    """Tests for the transmit_item() function."""

    def setUp(self):
        self.func_under_test = transmit_item

    @mock.patch("superdesk.publish.publish_content.get_resource_service")
    def test_marks_items_as_retrying_in_case_of_failure(self, *mocks):
        fake_get_service = mocks[0]
        fake_get_service().patch.side_effect = Exception("Error patching item")

        item_1 = {
            "_id": "item_1",
            "destination": {},
            "subscriber_id": "1",
            "item_id": "test",
            "headline": "test headline",
            "item_version": 4,
            "state": "pending",
        }

        orig_item = item_1.copy()  # item's original state in DB
        fake_get_service().find_one.return_value = orig_item

        with self.assertRaises(Exception):
            self.func_under_test(item_1["_id"])
        fake_get_service().system_update.assert_called_with(
            "item_1",
            {"_updated": ANY, "retry_attempt": 1, "state": "retrying", "next_retry_attempt_at": ANY},
            orig_item,
        )

    @mock.patch("superdesk.publish.publish_content.get_resource_service")
    def test_marks_items_as_retrying_second_time_incase_of_failure(self, *mocks):

        fake_get_service = mocks[0]
        fake_get_service().patch.side_effect = Exception("Error patching item")

        item_1 = {
            "_id": "item_1",
            "destination": {},
            "subscriber_id": "1",
            "item_id": "test",
            "headline": "test headline",
            "item_version": 4,
            "state": "retrying",
            "retry_attempt": 1,
            "next_retry_attempt_at": utcnow() - timedelta(minutes=10),
        }

        orig_item = item_1.copy()  # item's original state in DB
        fake_get_service().find_one.return_value = orig_item
        with self.assertRaises(Exception):
            self.func_under_test(item_1["_id"])

        fake_get_service().system_update.assert_called_with(
            "item_1",
            {"_updated": ANY, "retry_attempt": 2, "state": "retrying", "next_retry_attempt_at": ANY},
            orig_item,
        )

    @mock.patch("superdesk.publish.publish_content.get_resource_service")
    def test_marks_items_failed_to_transmit_after_all_retry_attempts(self, *mocks):

        fake_get_service = mocks[0]
        fake_get_service().patch.side_effect = Exception("Error patching item")
        self.app.config["MAX_TRANSMIT_RETRY_ATTEMPT"] = 4

        item_1 = {
            "_id": "item_1",
            "destination": {},
            "subscriber_id": "1",
            "item_id": "test",
            "headline": "test headline",
            "item_version": 4,
            "state": "retrying",
            "retry_attempt": 4,
            "next_retry_attempt_at": utcnow() - timedelta(minutes=10),
        }

        orig_item = item_1.copy()  # item's original state in DB
        fake_get_service().find_one.return_value = orig_item
        with self.assertRaises(Exception):
            self.func_under_test(item_1["_id"])
        fake_get_service().system_update.assert_called_with("item_1", {"_updated": ANY, "state": "failed"}, orig_item)

    @mock.patch("superdesk.publish.publish_content.logger")
    @mock.patch("superdesk.publish.publish_content.get_resource_service")
    def test_logs_error_even_when_marking_failed_items_fails(self, *mocks):
        fake_get_service = mocks[0]
        fake_get_service().patch.side_effect = Exception("Error patching item")
        fake_get_service().system_update.side_effect = Exception("Update error")

        item_1 = {
            "_id": "item_1",
            "subscriber_id": "1",
            "destination": {},
            "item_id": "test",
            "headline": "test headline",
            "item_version": 4,
            "state": "pending",
        }

        fake_get_service().find_one.return_value = item_1
        with self.assertRaises(Exception):
            self.func_under_test(item_1["_id"])
        fake_logger = mocks[1]
        expected_msg = "Failed to set the state for failed publish queue item item_1."
        fake_logger.error.assert_any_call(expected_msg)

    def test_transmit_failure(self):
        subscriber = {
            "_id": ObjectId("56c11bd78b84bb00b0a1905e"),
            "sequence_num_settings": {"max": 9999, "min": 1},
            "_etag": "f16b7eaa566f68b8d2561c811ec694bdf819784d",
            "is_active": True,
            "destinations": [{"delivery_type": "email"}],
            "email": "test@test.com",
            "subscriber_type": "digital",
            "name": "Test",
        }

        self.app.data.insert("subscribers", [subscriber])

        item_1 = {
            "_id": ObjectId(),
            "state": "pending",
            "item_id": "item_1",
            "item_version": 4,
            "headline": "pending headline",
            "destination": {"delivery_type": "email"},
            "subscriber_id": subscriber["_id"],
            "formatted_item": "test",
        }

        self.app.data.insert("publish_queue", [item_1])
        self.assertIsNone(self.func_under_test(item_1["_id"]))
        failed_item = self.app.data.find_one("publish_queue", req=None, _id=item_1["_id"])
        self.assertEqual(failed_item["state"], "retrying")
        self.assertEqual(failed_item["retry_attempt"], 1)
        self.assertEqual(failed_item["next_retry_attempt_at"], ANY)

    @mock.patch("superdesk.publish.publish_content.get_resource_service")
    @mock.patch("superdesk.publish.registered_transmitters")
    def test_no_retry_on_http_push_client_error(self, *mocks):
        self.app.config["MAX_TRANSMIT_RETRY_ATTEMPT"] = 4

        item_1 = {
            "_id": "item_1",
            "destination": {},
            "subscriber_id": "1",
            "item_id": "test",
            "headline": "test headline",
            "item_version": 4,
            "state": "pending",
        }

        orig_item = item_1.copy()  # item's original state in DB

        fake_get_service = mocks[1]
        fake_get_service().find_one.return_value = orig_item

        fake_transmitter = MagicMock()
        fake_transmitter.transmit.side_effect = PublishHTTPPushClientError.httpPushError(Exception("client 4xx"))

        fake_transmitters_list = mocks[0]
        fake_transmitters_list.__getitem__.return_value = fake_transmitter

        self.assertIsNone(self.func_under_test(item_1["_id"]))

        fake_get_service().system_update.assert_called_with("item_1", {"_updated": ANY, "state": "failed"}, orig_item)

    @mock.patch("superdesk.publish.publish_content.get_resource_service")
    @mock.patch("superdesk.publish.registered_transmitters")
    def test_retry_on_http_push_server_error(self, *mocks):
        self.app.config["MAX_TRANSMIT_RETRY_ATTEMPT"] = 4

        item_1 = {
            "_id": "item_1",
            "destination": {},
            "subscriber_id": "1",
            "item_id": "test",
            "headline": "test headline",
            "item_version": 4,
            "state": "pending",
        }

        orig_item = item_1.copy()  # item's original state in DB

        fake_get_service = mocks[1]
        fake_get_service().find_one.return_value = orig_item

        fake_transmitter = MagicMock()
        fake_transmitter.transmit.side_effect = PublishHTTPPushServerError.httpPushError(Exception("server 5xx"))

        fake_transmitters_list = mocks[0]
        fake_transmitters_list.__getitem__.return_value = fake_transmitter

        self.assertIsNone(self.func_under_test(item_1["_id"]))

        fake_get_service().system_update.assert_called_with(
            "item_1",
            {"_updated": ANY, "retry_attempt": 1, "state": "retrying", "next_retry_attempt_at": ANY},
            orig_item,
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import errors
from superdesk.errors import IngestApiError, IngestFileError, ParserError, ProviderError, IngestFtpError
from superdesk.tests import TestCase, setup_notification
from nose.tools import assert_raises
from superdesk.tests import setup
import logging
import unittest


class MockLoggingHandler(logging.Handler):
    """Mock logging handler to check for expected logs."""

    def __init__(self, *args, **kwargs):
        self.reset()
        logging.Handler.__init__(self, *args, **kwargs)

    def emit(self, record):
        self.messages[record.levelname.lower()].append(record.getMessage())

    def reset(self):
        self.messages = {
            "debug": [],
            "info": [],
            "warning": [],
            "error": [],
            "critical": [],
        }


class SuperdeskErrorTestCase(unittest.TestCase):
    """Tests for the `superdesk.errors.SuperdeskError` class."""

    def _get_target_class(self):
        """Return the class under test.

        Make the test fail immediately if the class cannot be imported.
        """
        try:
            from superdesk.errors import SuperdeskError
        except ImportError:
            self.fail("Could not import class under test (SuperdeskError).")
        else:
            return SuperdeskError

    def test_sets_error_description_to_none_by_default(self):
        klass = self._get_target_class()
        instance = klass(1234)
        self.assertIsNone(instance.desc)

    def test_stores_given_detailed_error_description(self):
        klass = self._get_target_class()
        instance = klass(1234, desc="Detailed error description.")
        self.assertEqual(instance.desc, "Detailed error description.")

    def test_string_representation_of_the_instance_no_description(self):
        klass = self._get_target_class()
        instance = klass(1234)
        instance.code = 101
        instance.message = "Foobar error"
        instance.desc = None

        self.assertEqual(str(instance), "SuperdeskError Error 101 - Foobar error")

    def test_string_representation_of_the_instance_description_given(self):
        klass = self._get_target_class()
        instance = klass(1234)
        instance.code = 101
        instance.message = "Foobar error"
        instance.desc = "This is a detailed description"

        self.assertEqual(
            str(instance), ("SuperdeskError Error 101 - Foobar error " "Details: This is a detailed description")
        )


class ErrorsTestCase(TestCase):

    mock_logger_handler = {}

    def setUp(self):
        setup(context=self)
        setup_notification(context=self)
        mock_logger = logging.getLogger("test")
        self.mock_logger_handler = MockLoggingHandler()
        mock_logger.addHandler(self.mock_logger_handler)
        errors.logger = mock_logger
        errors.notifiers = []
        self.provider = {"name": "TestProvider"}

    def test_raise_apiGeneralError(self):
        with assert_raises(IngestApiError) as error_context:
            try:
                ex = Exception("Testing general API error")
                raise ex
            except Exception:
                raise IngestApiError.apiGeneralError(ex, self.provider)

        exception = error_context.exception
        self.assertEqual(exception.code, 4000)
        self.assertEqual(exception.message, "Unknown API ingest error")
        self.assertEqual(exception.provider_name, "TestProvider")

        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing general API error")

        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestApiError Error 4000 - Unknown API ingest error: "
            "Testing general API error on channel TestProvider",
        )

    def test_raise_apiRequestError(self):
        with assert_raises(IngestApiError) as error_context:
            try:
                ex = Exception("Testing apiRequestError")
                raise ex
            except Exception:
                raise IngestApiError.apiRequestError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 4003)
        self.assertTrue(exception.message == "API ingest has request error")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing apiRequestError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestApiError Error 4003 - API ingest has request error: "
            "Testing apiRequestError on channel TestProvider",
        )

    def test_raise_apiTimeoutError(self):
        with assert_raises(IngestApiError) as error_context:
            try:
                ex = Exception("Testing apiTimeoutError")
                raise ex
            except Exception:
                raise IngestApiError.apiTimeoutError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 4001)
        self.assertTrue(exception.message == "API ingest connection has timed out.")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing apiTimeoutError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestApiError Error 4001 - API ingest connection has timed out.: "
            "Testing apiTimeoutError on channel TestProvider",
        )

    def test_raise_apiRedirectError(self):
        with assert_raises(IngestApiError) as error_context:
            try:
                ex = Exception("Testing apiRedirectError")
                raise ex
            except Exception:
                raise IngestApiError.apiRedirectError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 4002)
        self.assertTrue(exception.message == "API ingest has too many redirects")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing apiRedirectError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestApiError Error 4002 - API ingest has too many redirects: "
            "Testing apiRedirectError on channel TestProvider",
        )

    def test_raise_apiUnicodeError(self):
        with assert_raises(IngestApiError) as error_context:
            try:
                ex = Exception("Testing apiUnicodeError")
                raise ex
            except Exception:
                raise IngestApiError.apiUnicodeError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 4004)
        self.assertTrue(exception.message == "API ingest Unicode Encode Error")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing apiUnicodeError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestApiError Error 4004 - API ingest Unicode Encode Error: "
            "Testing apiUnicodeError on channel TestProvider",
        )

    def test_raise_apiParseError(self):
        with assert_raises(IngestApiError) as error_context:
            try:
                ex = Exception("Testing apiParseError")
                raise ex
            except Exception:
                raise IngestApiError.apiParseError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 4005)
        self.assertTrue(exception.message == "API ingest xml parse error")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing apiParseError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestApiError Error 4005 - API ingest xml parse error: " "Testing apiParseError on channel TestProvider",
        )

    def test_raise_apiNotFoundError(self):
        with assert_raises(IngestApiError) as error_context:
            try:
                ex = Exception("Testing apiNotFoundError")
                raise ex
            except Exception:
                raise IngestApiError.apiNotFoundError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 4006)
        self.assertTrue(exception.message == "API service not found(404) error")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing apiNotFoundError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestApiError Error 4006 - API service not found(404) error: "
            "Testing apiNotFoundError on channel TestProvider",
        )

    def test_raise_apiAuthError(self):
        with assert_raises(IngestApiError) as error_context:
            try:
                ex = Exception("Testing API authorization error")
                raise ex
            except Exception:
                raise IngestApiError.apiAuthError(ex, self.provider)

        exception = error_context.exception
        self.assertEqual(exception.code, 4007)
        self.assertEqual(exception.message, "API authorization error")
        self.assertEqual(exception.provider_name, "TestProvider")

        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing API authorization error")

        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestApiError Error 4007 - API authorization error: "
            "Testing API authorization error on channel TestProvider",
        )

    def test_raise_folderCreateError(self):
        with assert_raises(IngestFileError) as error_context:
            try:
                ex = Exception("Testing folderCreateError")
                raise ex
            except Exception:
                raise IngestFileError.folderCreateError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 3001)
        self.assertTrue(exception.message == "Destination folder could not be created")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing folderCreateError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestFileError Error 3001 - Destination folder could not be created: "
            "Testing folderCreateError on channel TestProvider",
        )

    def test_raise_fileMoveError(self):
        with assert_raises(IngestFileError) as error_context:
            try:
                ex = Exception("Testing fileMoveError")
                raise ex
            except Exception:
                raise IngestFileError.fileMoveError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 3002)
        self.assertTrue(exception.message == "Ingest file could not be copied")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing fileMoveError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestFileError Error 3002 - Ingest file could not be copied: "
            "Testing fileMoveError on channel TestProvider",
        )

    def test_raise_parseMessageError(self):
        with assert_raises(ParserError) as error_context:
            try:
                ex = Exception("Testing parseMessageError")
                raise ex
            except Exception:
                raise ParserError.parseMessageError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 1001)
        self.assertTrue(exception.message == "Message could not be parsed")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing parseMessageError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ParserError Error 1001 - Message could not be parsed: "
            "Testing parseMessageError on channel TestProvider",
        )

    def test_parse_message_error_save_data(self):
        data = "some data"
        with assert_raises(ParserError):
            try:
                raise Exception("Err message")
            except Exception as ex:
                raise ParserError.parseMessageError(ex, self.provider, data=data)
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        message = self.mock_logger_handler.messages["error"][0]
        self.assertIn("file=", message)
        filename = message.split("file=")[1]
        with open(filename, "r") as file:
            self.assertEqual(data, file.read())

    def test_raise_parseFileError(self):
        with assert_raises(ParserError) as error_context:
            try:
                raise Exception("Testing parseFileError")
            except Exception as ex:
                raise ParserError.parseFileError("afp", "test.txt", ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 1002)
        self.assertTrue(exception.message == "Ingest file could not be parsed")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing parseFileError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        message = self.mock_logger_handler.messages["error"][0]
        self.assertIn("ParserError Error 1002 - Ingest file could not be parsed", message)
        self.assertIn("Testing parseFileError on channel TestProvider", message)
        self.assertIn("source=afp", message)
        self.assertIn("file=test.txt", message)

    def test_raise_newsmlOneParserError(self):
        with assert_raises(ParserError) as error_context:
            try:
                raise Exception("Testing newsmlOneParserError")
            except Exception as ex:
                raise ParserError.newsmlOneParserError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 1004)
        self.assertTrue(exception.message == "NewsML1 input could not be processed")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing newsmlOneParserError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ParserError Error 1004 - NewsML1 input could not be processed: "
            "Testing newsmlOneParserError on channel TestProvider",
        )

    def test_raise_newsmlTwoParserError(self):
        with assert_raises(ParserError) as error_context:
            try:
                ex = Exception("Testing newsmlTwoParserError")
                raise ex
            except Exception:
                raise ParserError.newsmlTwoParserError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 1005)
        self.assertTrue(exception.message == "NewsML2 input could not be processed")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing newsmlTwoParserError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ParserError Error 1005 - NewsML2 input could not be processed: "
            "Testing newsmlTwoParserError on channel TestProvider",
        )

    def test_raise_nitfParserError(self):
        with assert_raises(ParserError) as error_context:
            try:
                ex = Exception("Testing nitfParserError")
                raise ex
            except Exception:
                raise ParserError.nitfParserError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 1006)
        self.assertTrue(exception.message == "NITF input could not be processed")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing nitfParserError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ParserError Error 1006 - NITF input could not be processed: "
            "Testing nitfParserError on channel TestProvider",
        )

    def test_raise_providerAddError(self):
        with assert_raises(ProviderError) as error_context:
            try:
                ex = Exception("Testing providerAddError")
                raise ex
            except Exception:
                raise ProviderError.providerAddError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 2001)
        self.assertTrue(exception.message == "Provider could not be saved")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing providerAddError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ProviderError Error 2001 - Provider could not be saved: "
            "Testing providerAddError on channel TestProvider",
        )

    def test_raise_expiredContentError(self):
        with assert_raises(ProviderError) as error_context:
            try:
                ex = Exception("Testing expiredContentError")
                raise ex
            except Exception:
                raise ProviderError.expiredContentError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 2002)
        self.assertTrue(exception.message == "Expired content could not be removed")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing expiredContentError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ProviderError Error 2002 - Expired content could not be removed: "
            "Testing expiredContentError on channel TestProvider",
        )

    def test_raise_ruleError(self):
        with assert_raises(ProviderError) as error_context:
            try:
                ex = Exception("Testing ruleError")
                raise ex
            except Exception:
                raise ProviderError.ruleError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 2003)
        self.assertTrue(exception.message == "Rule could not be applied")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing ruleError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ProviderError Error 2003 - Rule could not be applied: " "Testing ruleError on channel TestProvider",
        )

    def test_raise_ingestError(self):
        with assert_raises(ProviderError) as error_context:
            try:
                ex = Exception("Testing ingestError")
                raise ex
            except Exception:
                raise ProviderError.ingestError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 2004)
        self.assertTrue(exception.message == "Ingest error")
        self.assertTrue(exception.provider_name == "TestProvider")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing ingestError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ProviderError Error 2004 - Ingest error: " "Testing ingestError on channel TestProvider",
        )

    def test_raise_anpaError(self):
        with assert_raises(ProviderError) as error_context:
            try:
                ex = Exception("Testing anpaError")
                raise ex
            except Exception:
                raise ProviderError.anpaError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 2005)
        self.assertTrue(exception.message == "Anpa category error")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing anpaError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ProviderError Error 2005 - Anpa category error: " "Testing anpaError on channel TestProvider",
        )

    def test_raise_providerFilterExpiredContentError(self):
        with assert_raises(ProviderError) as error_context:
            try:
                ex = Exception("Testing providerFilterExpiredContentError")
                raise ex
            except Exception:
                raise ProviderError.providerFilterExpiredContentError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 2006)
        self.assertTrue(exception.message == "Expired content could not be filtered")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing providerFilterExpiredContentError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "ProviderError Error 2006 - Expired content could not be filtered: "
            "Testing providerFilterExpiredContentError on channel TestProvider",
        )

    def test_raise_ftpError(self):
        with assert_raises(IngestFtpError) as error_context:
            try:
                ex = Exception("Testing ftpError")
                raise ex
            except Exception:
                raise IngestFtpError.ftpError(ex, self.provider)
        exception = error_context.exception
        self.assertTrue(exception.code == 5000)
        self.assertTrue(exception.message == "FTP ingest error")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing ftpError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestFtpError Error 5000 - FTP ingest error: " "Testing ftpError on channel TestProvider",
        )

    def test_raise_ftpUnknownParserError(self):
        with assert_raises(IngestFtpError) as error_context:
            try:
                raise Exception("Testing ftpUnknownParserError")
            except Exception as ex:
                raise IngestFtpError.ftpUnknownParserError(ex, self.provider, "test.xml")
        exception = error_context.exception
        self.assertTrue(exception.code == 5001)
        self.assertTrue(exception.message == "FTP parser could not be found")
        self.assertIsNotNone(exception.system_exception)
        self.assertEqual(exception.system_exception.args[0], "Testing ftpUnknownParserError")
        self.assertEqual(len(self.mock_logger_handler.messages["error"]), 1)
        self.assertEqual(
            self.mock_logger_handler.messages["error"][0],
            "IngestFtpError Error 5001 - FTP parser could not be found: "
            "Testing ftpUnknownParserError on channel TestProvider file=test.xml",
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from abc import abstractclassmethod
from string import Template
from types import ModuleType
from flask import current_app
import superdesk
import getpass
import os
import re
import time
from eve.utils import ParsedRequest


DEFAULT_DATA_UPDATE_DIR_NAME = "data_updates"
MAIN_DATA_UPDATES_DIR = os.path.abspath(
    os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir, DEFAULT_DATA_UPDATE_DIR_NAME)
)
DATA_UPDATES_FILENAME_REGEX = re.compile("^(\d+).*\.py$")
DATA_UPDATE_TEMPLATE = """
# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : $user
# Creation: $current_date

from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = '$resource'

    def forwards(self, mongodb_collection, mongodb_database):
        $default_fw_implementation

    def backwards(self, mongodb_collection, mongodb_database):
        $default_bw_implementation
""".lstrip(
    "\n"
)
DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION = "raise NotImplementedError()"
DEFAULT_DATA_UPDATE_BW_IMPLEMENTATION = "raise NotImplementedError()"


def get_dirs(only_relative_folder=False):
    dirs = []
    try:
        with superdesk.app.app_context():
            dirs.append(current_app.config.get("DATA_UPDATES_PATH", DEFAULT_DATA_UPDATE_DIR_NAME))
            if current_app.config.get("APPS_DATA_UPDATES_PATHS"):
                dirs.extend(current_app.config["APPS_DATA_UPDATES_PATHS"])
    except RuntimeError:
        # working outside of application context
        pass
    if not only_relative_folder:
        dirs.append(MAIN_DATA_UPDATES_DIR)
    assert len(dirs)
    return dirs


def get_data_updates_files(strip_file_extension=False):
    """Return the list of data updates filenames.

    .py extension can be removed with `strip_file_extension` parameter.
    """
    files = []
    # create folder if doens't exist
    for folder in get_dirs():
        if not os.path.exists(folder):
            continue
        # list all files from data updates directory
        if os.path.exists(folder):
            files += [f for f in os.listdir(folder) if os.path.isfile(os.path.join(folder, f))]
    # keep only data updates (00000x*.py)
    files = [f for f in files if DATA_UPDATES_FILENAME_REGEX.match(f)]
    # order file names
    files = sorted(files)
    # remove .py extention if needed
    if strip_file_extension:
        files = [f.rstrip(".py") for f in files]
    return files


class DataUpdateCommand(superdesk.Command):
    """Parent class for Upgrade and Downgrade commands.

    It defines options and initialize some variables in `run` method.
    """

    option_list = [
        superdesk.Option(
            "--id",
            "-i",
            dest="data_update_id",
            required=False,
            choices=get_data_updates_files(strip_file_extension=True),
            help="Data update id to run last",
        ),
        superdesk.Option(
            "--fake-init",
            dest="fake",
            required=False,
            action="store_true",
            help="Mark data updates as run without actually running them",
        ),
        superdesk.Option(
            "--dry-run",
            dest="dry",
            required=False,
            action="store_true",
            help="Does not mark data updates as done. This can be useful for development.",
        ),
    ]

    def get_applied_updates(self):
        req = ParsedRequest()
        req.sort = "-name"
        return tuple(self.data_updates_service.get(req=req, lookup={}))

    def run(self, data_update_id=None, fake=False, dry=False):
        self.data_updates_service = superdesk.get_resource_service("data_updates")
        self.data_updates_files = get_data_updates_files(strip_file_extension=True)
        # retrieve existing data updates in database
        data_updates_applied = self.get_applied_updates()
        self.last_data_update = data_updates_applied and data_updates_applied[-1] or None
        if self.last_data_update:
            if self.last_data_update["name"] not in self.data_updates_files:
                print(
                    "A data update previously applied to this database (%s) can't be found in %s"
                    % (self.last_data_update["name"], ", ".join(get_dirs()))
                )

    def compile_update_in_module(self, data_update_name):
        date_update_script_file = None
        for folder in get_dirs():
            date_update_script_file = os.path.join(folder, "%s.py" % (data_update_name))
            if os.path.exists(date_update_script_file):
                break
        assert date_update_script_file is not None, "File %s has not been found" % (data_update_name)
        # create a module instance to use as scope for our data update
        module = ModuleType("data_update_module")
        with open(date_update_script_file) as f:
            # compile data update script file
            script = compile(f.read(), date_update_script_file, "exec")
            # excecute the script in the module
            exec(script, module.__dict__)
        return module

    def in_db(self, update):
        return update in map(lambda _: _["name"], self.get_applied_updates())


class Upgrade(DataUpdateCommand):
    """Runs all the new data updates available.

    If ``data_update_id`` is given, runs new data updates until the given one.

    Example:
    ::

        $ python manage.py data:upgrade

    """

    def run(self, data_update_id=None, fake=False, dry=False):
        super().run(data_update_id, fake, dry)
        data_updates_files = self.data_updates_files
        # drops updates that already have been applied
        data_updates_files = [update for update in data_updates_files if not self.in_db(update)]
        # drop versions after given one
        if data_update_id:
            if data_update_id not in data_updates_files:
                print("Given data update id not found in available updates. It may have been already applied")
                return False
            data_updates_files = data_updates_files[: data_updates_files.index(data_update_id) + 1]
        # apply data updates
        for data_update_name in data_updates_files:
            print("data update %s running forward..." % (data_update_name))
            module_scope = self.compile_update_in_module(data_update_name)
            # run the data update forward
            if not fake:
                module_scope.DataUpdate().apply("forwards")
            if not dry:
                # store the applied data update in the database
                self.data_updates_service.create([{"name": data_update_name}])
        if not data_updates_files:
            print("No data update to apply.")


class Downgrade(DataUpdateCommand):
    """Runs the latest data update backward.

    If ``data_update_id`` is given, runs all the data updates backward until the given one.

    Example:
    ::

        $ python manage.py data:downgrade

    """

    def run(self, data_update_id=None, fake=False, dry=False):
        super().run(data_update_id, fake, dry)
        data_updates_files = self.data_updates_files
        # check if there is something to downgrade
        if not self.last_data_update:
            print("No data update has been already applied")
            return False
        # drops updates which have not been already made (this is rollback mode)
        data_updates_files = [update for update in data_updates_files if self.in_db(update)]

        # if data_update_id is given, go until this update (drop previous updates)
        if data_update_id:
            if data_update_id not in data_updates_files:
                print("Update %s can't be find. It may have been already downgraded" % (data_update_id))
                return False
            data_updates_files = data_updates_files[data_updates_files.index(data_update_id) :]
        # otherwise, just rollback one update
        else:
            print(
                "No data update id has been provided. Dowgrading to previous version: %s"
                % self.last_data_update["name"]
            )
            data_updates_files = data_updates_files[len(data_updates_files) - 1 :]
        # apply data updates, in the opposite direction
        for data_update_name in reversed(data_updates_files):
            print("data update %s running backward..." % (data_update_name))
            module_scope = self.compile_update_in_module(data_update_name)
            # run the data update backward
            if not fake:
                module_scope.DataUpdate().apply("backwards")
            if not dry:
                # remove the applied data update from the database
                self.data_updates_service.delete({"name": data_update_name})
        if not data_updates_files:
            print("No data update to apply.")


class GenerateUpdate(superdesk.Command):
    """Generate a file where to define a new data update.

    Example:
    ::

        $ python manage.py data:generate_update --resource=archive

    """

    option_list = [
        superdesk.Option("--resource", "-r", dest="resource_name", required=True, help="Resource to update"),
        superdesk.Option(
            "--global",
            "-g",
            dest="global_update",
            required=False,
            action="store_true",
            help="This data update belongs to superdesk core",
        ),
    ]

    def run(self, resource_name, global_update=False):
        timestamp = time.strftime("%Y%m%d-%H%M%S")
        # create a data update file
        try:
            last_file = get_data_updates_files()[-1]
            name_id = int(last_file.replace(".py", "").split("_")[0]) + 1
        except IndexError:
            name_id = 0
        if global_update:
            update_dir = MAIN_DATA_UPDATES_DIR
        else:
            update_dir = get_dirs(only_relative_folder=True)[0]
        if not os.path.exists(update_dir):
            os.makedirs(update_dir)
        data_update_filename = os.path.join(update_dir, "{:05d}_{}_{}.py".format(name_id, timestamp, resource_name))
        if os.path.exists(data_update_filename):
            raise Exception('The file "%s" already exists' % (data_update_filename))
        with open(data_update_filename, "w+") as f:
            template_context = {
                "resource": resource_name,
                "current_date": time.strftime("%Y-%m-%d %H:%M"),
                "user": getpass.getuser(),
                "default_fw_implementation": DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION,
                "default_bw_implementation": DEFAULT_DATA_UPDATE_BW_IMPLEMENTATION,
            }
            f.write(Template(DATA_UPDATE_TEMPLATE).substitute(template_context))
            print("Data update file created %s" % (data_update_filename))


superdesk.command("data:generate_update", GenerateUpdate())
superdesk.command("data:upgrade", Upgrade())
superdesk.command("data:downgrade", Downgrade())


class BaseDataUpdate:
    def apply(self, direction):
        assert direction in ["forwards", "backwards"]
        collection = current_app.data.get_mongo_collection(self.resource)
        db = current_app.data.driver.db
        getattr(self, direction)(collection, db)


DataUpdate = BaseDataUpdate



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from superdesk.resource import Resource

logger = logging.getLogger(__name__)


class LanguagesResource(Resource):
    schema = {
        "language": {"type": "string", "required": True, "unique": True},
        "label": {"type": "string", "required": True, "unique": True},
        "source": {"type": "boolean", "default": False},
        "destination": {"type": "boolean", "default": False},
    }

    resource_methods = ["GET"]
    item_methods = []



from flask import json
from behave import given, when, then  # @UnresolvedImport
from superdesk import get_resource_service
from superdesk.tests import get_prefixed_url
from superdesk.utc import utcnow
from datetime import timedelta
from features.steps.steps import apply_placeholders


@given("highlights")
def given_highlights(context):
    with context.app.app_context():
        context.desks = {"name": "test"}
        get_resource_service("desks").post([context.desks])
        context.highlights = {"name": "highlight", "desks": [context.desks["_id"]], "auto_insert": "now-12h"}
        get_resource_service("highlights").post([context.highlights])
        task = {"desk": context.desks["_id"]}
        context.items = [
            {
                "headline": "item1",
                "state": "in_progress",
                "task": task,
                "versioncreated": utcnow() - timedelta(minutes=5),
            },
            {
                "headline": "item2",
                "state": "in_progress",
                "task": task,
                "versioncreated": utcnow() - timedelta(hours=8),
            },
            {"headline": "old", "state": "in_progress", "task": task, "versioncreated": utcnow() - timedelta(days=2)},
        ]
        get_resource_service("archive").post(context.items)
        for item in context.items:
            marks = [{"highlights": context.highlights["_id"], "marked_item": item["_id"]}]
            get_resource_service("marked_for_highlights").post(marks)


@when("we create highlights package")
def when_we_create_highglights_package(context):
    data_text = (
        '{"highlight": "%s", "type": "composite", '
        '"task": {"user": "#user._id#", "desk": "#desks._id#"}}' % str(context.highlights["_id"])
    )
    data_text = apply_placeholders(context, data_text)
    url = get_prefixed_url(context.app, "/archive")
    context.response = context.client.post(url, data=data_text, headers=context.headers)


@then("we get new package with items")
def then_we_get_new_package_with_items(context):
    assert context.response.status_code == 201, "%d: %s" % (context.response.status_code, context.response.get_data())
    package = json.loads(context.response.get_data())

    groups = package.get("groups")
    assert len(groups) == 2, "there should be 2 groups"

    refs = groups[1].get("refs")
    assert len(refs) == 2, "there should be 2 refs %s" % (refs)
    assert refs[0]["headline"] == "item1", refs[0]
    assert refs[1]["headline"] == "item2", refs[1]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging
import superdesk

from copy import copy

from eve.utils import ParsedRequest
from eve.versioning import resolve_document_version

from superdesk.users.services import current_user_has_privilege
from superdesk.resource import Resource
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.notification import push_notification
from superdesk.utc import utcnow
from superdesk.metadata.utils import item_url
from superdesk.services import BaseService
from superdesk.metadata.item import metadata_schema, ITEM_STATE, CONTENT_STATE, ITEM_TYPE
from superdesk import get_resource_service, config
from superdesk.activity import add_activity, ACTIVITY_CREATE, ACTIVITY_UPDATE
from superdesk.workflow import is_workflow_state_transition_valid
from apps.archive.common import (
    get_item_expiry,
    item_operations,
    ITEM_OPERATION,
    update_version,
    insert_into_versions,
    is_assigned_to_a_desk,
    convert_task_attributes_to_objectId,
    on_create_item,
    ARCHIVE,
    get_subject,
)
from flask_babel import _

task_statuses = ["todo", "in_progress", "done"]
default_status = "todo"
ITEM_SEND = "send"
item_operations.append(ITEM_SEND)
MACRO_INCOMING = "incoming"
MACRO_OUTGOING = "outgoing"
MACRO_ONSTAGE = "onstage"

logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = "tasks"
    service = TasksService(TaskResource.datasource["source"], backend=superdesk.get_backend())
    TaskResource(endpoint_name, app=app, service=service)


def compare_dictionaries(original, updates):
    original_keys = set(original.keys())
    updates_keys = set(updates.keys())
    intersect_keys = original_keys.intersection(updates_keys)
    added = list(updates_keys - original_keys)
    modified = [o for o in intersect_keys if original[o] != updates[o]]
    modified.extend(added)
    return modified


def send_to(doc, update=None, desk_id=None, stage_id=None, user_id=None, default_stage="incoming_stage"):
    """Send item to given desk and stage.

    Applies the outgoing and incoming macros of current and destination stages

    :param doc: original document to be sent
    :param update: updates for the document
    :param desk: id of desk where item should be sent
    :param stage: optional stage within the desk
    :param default_stage: if no stage_id is passed then it determines the stage in that desk the doc is assigned,
            either the the incomming stage or the working stage.
    """

    original_task = doc.setdefault("task", {})
    current_stage = None
    if original_task.get("stage"):
        current_stage = get_resource_service("stages").find_one(req=None, _id=original_task.get("stage"))
    desk = destination_stage = None
    task = {"desk": desk_id, "stage": stage_id, "user": original_task.get("user") if user_id is None else user_id}

    if current_stage:
        apply_stage_rule(doc, update, current_stage, MACRO_OUTGOING)

    if desk_id:
        desk = superdesk.get_resource_service("desks").find_one(req=None, _id=desk_id)
        if not desk:
            raise SuperdeskApiError.notFoundError(_("Invalid desk identifier {desk_id}").format(desk_id=desk_id))

        if not current_user_has_privilege("move") and str(user_id) not in [
            str(x.get("user", "")) for x in desk.get("members", [])
        ]:
            raise SuperdeskApiError.forbiddenError(_("User is not member of desk: {desk_id}").format(desk_id=desk_id))

        task["desk"] = desk_id
        if not stage_id:
            task["stage"] = desk.get(default_stage)
            destination_stage = get_resource_service("stages").find_one(req=None, _id=desk.get(default_stage))

    if stage_id:
        destination_stage = get_resource_service("stages").find_one(req=None, _id=stage_id)
        if not destination_stage:
            raise SuperdeskApiError.notFoundError(_("Invalid stage identifier {stage_id}").format(stage_id=stage_id))

        task["desk"] = destination_stage["desk"]
        task["stage"] = stage_id

    if destination_stage:
        apply_stage_rule(doc, update, destination_stage, MACRO_INCOMING, desk=desk, task=task)
        if destination_stage.get("task_status"):
            task["status"] = destination_stage["task_status"]

    if update:
        update.setdefault("task", {})
        update["task"].update(task)
        update["expiry"] = get_item_expiry(desk=desk, stage=destination_stage)
    else:
        doc["task"].update(task)
        doc["expiry"] = get_item_expiry(desk=desk, stage=destination_stage)
        superdesk.get_resource_service("desks").apply_desk_metadata(doc, doc)


def apply_stage_rule(doc, update, stage, rule_type, desk=None, task=None):
    macro_type = "{}_macro".format(rule_type)

    if stage.get(macro_type):
        try:
            original_doc = dict(doc)
            macro = get_resource_service("macros").get_macro_by_name(stage.get(macro_type))
            if not macro:
                logger.warning("macro %s is missing", stage.get(macro_type))
                return
            macro["callback"](doc, desk=desk, stage=stage, task=task)
            if update:
                modified = compare_dictionaries(original_doc, doc)
                for i in modified:
                    update[i] = doc[i]
        except Exception as ex:
            message = _("Error:{exception} in {rule_type} rule:{rule} for stage:{stage}").format(
                exception=str(ex), rule_type=rule_type, rule=macro.get("label"), stage=stage.get("name")
            )
            raise SuperdeskApiError.badRequestError(message)


def apply_onstage_rule(doc, _id):
    """Apply any on stage macro/rule that may be defined for the stage.

    :param doc:
    :param _id:
    :return:
    """
    doc[config.ID_FIELD] = _id
    stage = get_resource_service("stages").find_one(req=None, _id=doc.get("task", {}).get("stage"))
    if stage:
        apply_stage_rule(doc, None, stage, "onstage")


class TaskResource(Resource):
    datasource = {
        "source": "archive",
        "default_sort": [("_updated", -1)],
        "filter": {"task": {"$exists": True}},
        "elastic_filter": {
            "bool": {
                "must": {"exists": {"field": "task"}},
                "must_not": {"term": {ITEM_STATE: "spiked"}},
            }
        },
    }

    item_url = item_url
    schema = {
        "expiry": {"type": "string"},
        "slugline": metadata_schema["slugline"],
        "description_text": metadata_schema["description_text"],
        "type": metadata_schema["type"],
        "task": {
            "type": "dict",
            "schema": {
                "status": {"type": "string", "allowed": task_statuses, "default": default_status},
                "due_date": {"type": "datetime"},
                "started_at": {"type": "datetime"},
                "finished_at": {"type": "datetime"},
                "user": Resource.rel("users", True),
                "desk": Resource.rel("desks", True),
                "stage": Resource.rel("stages", True),
            },
        },
    }
    privileges = {"POST": "tasks", "PATCH": "tasks", "DELETE": "tasks"}


class TasksService(BaseService):
    def get(self, req, lookup):
        if req is None:
            req = ParsedRequest()
        return self.backend.get("tasks", req=req, lookup=lookup)

    def update_times(self, doc):
        task = doc.get("task", {})
        status = task.get("status", None)
        if status == "in_progress":
            task.setdefault("started_at", utcnow())

        if status == "done":
            task.setdefault("finished_at", utcnow())

    def __is_content_moved_from_desk(self, doc):
        """
        Returns True if the 'doc' is being moved from a desk. False otherwise.
        """
        return doc.get("task", {}).get("desk", None) is None

    def __is_content_assigned_to_new_desk(self, original, updates):
        """Checks if the content is assigned to a new desk.

        :return: True if the content is being moved to a new desk. False otherwise.
        """
        return str(original.get("task", {}).get("desk", "")) != str(updates.get("task", {}).get("desk", ""))

    def __update_state(self, updates, original):
        if self.__is_content_assigned_to_new_desk(original, updates):
            # check if the preconditions for the action are in place
            original_state = original[ITEM_STATE]
            if not is_workflow_state_transition_valid("move", original_state):
                raise InvalidStateTransitionError()

            updates[ITEM_STATE] = (
                CONTENT_STATE.DRAFT if self.__is_content_moved_from_desk(updates) else CONTENT_STATE.SUBMITTED
            )
            resolve_document_version(updates, ARCHIVE, "PATCH", original)

    def update_stage(self, doc):
        task = doc.get("task", {})
        desk_id = task.get("desk", None)
        stage_id = task.get("stage", None)
        send_to(doc=doc, desk_id=desk_id, stage_id=stage_id)

    def on_create(self, docs):
        on_create_item(docs)
        for doc in docs:
            resolve_document_version(doc, ARCHIVE, "POST")
            self.update_times(doc)
            self.update_stage(doc)
            convert_task_attributes_to_objectId(doc)

    def on_created(self, docs):
        push_notification(self.datasource, created=1)
        push_notification("task:new")
        for doc in docs:
            insert_into_versions(doc["_id"])
            if is_assigned_to_a_desk(doc):
                add_activity(
                    ACTIVITY_CREATE,
                    "added new task {{ subject }} of type {{ type }}",
                    self.datasource,
                    item=doc,
                    subject=get_subject(doc),
                    type=doc[ITEM_TYPE],
                )

    def on_update(self, updates, original):
        self.update_times(updates)
        if is_assigned_to_a_desk(updates):
            self.__update_state(updates, original)
        new_stage_id = str(updates.get("task", {}).get("stage", ""))
        old_stage_id = str(original.get("task", {}).get("stage", ""))
        new_user_id = updates.get("task", {}).get("user", "")
        if new_stage_id and new_stage_id != old_stage_id:
            updates[ITEM_OPERATION] = ITEM_SEND
            send_to(doc=original, update=updates, desk_id=None, stage_id=new_stage_id, user_id=new_user_id)
            resolve_document_version(updates, ARCHIVE, "PATCH", original)
        convert_task_attributes_to_objectId(updates)
        update_version(updates, original)

    def on_updated(self, updates, original):
        updated = copy(original)
        updated.update(updates)
        if self._stage_changed(updates, original):
            insert_into_versions(doc=updated)
        new_task = updates.get("task", {})
        old_task = original.get("task", {})
        if new_task.get("stage") != old_task.get("stage"):
            push_notification(
                "task:stage",
                new_stage=str(new_task.get("stage", "")),
                old_stage=str(old_task.get("stage", "")),
                new_desk=str(new_task.get("desk", "")),
                old_desk=str(old_task.get("desk", "")),
            )
        else:
            push_notification(self.datasource, updated=1)

        if is_assigned_to_a_desk(updated):
            if self.__is_content_assigned_to_new_desk(original, updates) and not self._stage_changed(updates, original):
                insert_into_versions(doc=updated)
            add_activity(
                ACTIVITY_UPDATE,
                "updated task {{ subject }} for item {{ type }}",
                self.datasource,
                item=updated,
                subject=get_subject(updated),
                type=updated["type"],
            )

    def on_deleted(self, doc):
        push_notification(self.datasource, deleted=1)

    def assign_user(self, item_id, updates):
        return self.patch(item_id, updates)

    def _stage_changed(self, updates, original):
        new_stage_id = str(updates.get("task", {}).get("stage", ""))
        old_stage_id = str(original.get("task", {}).get("stage", ""))
        return new_stage_id and new_stage_id != old_stage_id


superdesk.privilege(name="tasks", label="Tasks Management", description="Tasks Management")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.io.registry import register_feeding_service, register_feeding_service_parser
from superdesk.io.feeding_services import FeedingService
from superdesk.errors import IngestApiError
import requests
import traceback

# from pytz import timezone


WUFOO_URL = "https://{subdomain}.wufoo.com/api/v3/"
WUFOO_QUERY_FORM = "forms/{form_hash}/"
WUFOO_QUERY_FIELDS = "fields.json"
WUFOO_QUERY_ENTRIES = "entries.json"


class WufooFeedingService(FeedingService):
    """
    Feeding Service class which can read article(s) using Wufoo API
    """

    NAME = "wufoo"

    ERRORS = [
        IngestApiError.apiTimeoutError().get_error_description(),
        IngestApiError.apiRedirectError().get_error_description(),
        IngestApiError.apiRequestError().get_error_description(),
        IngestApiError.apiGeneralError().get_error_description(),
    ]

    label = "Wufoo feed API"

    fields = [
        {"id": "wufoo_username", "type": "text", "label": "Login", "placeholder": "Wufoo login", "required": True},
        {
            "id": "wufoo_api_key",
            "type": "password",
            "label": "API key",
            "placeholder": "Wufoo API Key",
            "required": True,
        },
    ]

    def __init__(self):
        super().__init__()
        self.fields_cache = {}

    def _update(self, provider, update):
        user = provider["config"]["wufoo_username"]
        wufoo_data = {
            "url": WUFOO_URL.format(subdomain=user),
            "user": user,
            "api_key": provider["config"]["wufoo_api_key"],
            "form_query_entries_tpl": WUFOO_QUERY_FORM + WUFOO_QUERY_ENTRIES,
            "update": update,
        }
        try:
            parser = self.get_feed_parser(provider, None)
        except requests.exceptions.Timeout as ex:
            raise IngestApiError.apiTimeoutError(ex, provider)
        except requests.exceptions.TooManyRedirects as ex:
            raise IngestApiError.apiRedirectError(ex, provider)
        except requests.exceptions.RequestException as ex:
            raise IngestApiError.apiRequestError(ex, provider)
        except Exception as error:
            traceback.print_exc()
            raise IngestApiError.apiGeneralError(error, self.provider)
        items = parser.parse(wufoo_data, provider)
        return [items]


register_feeding_service(WufooFeedingService)
register_feeding_service_parser(WufooFeedingService.NAME, "wufoo")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource, text_with_keyword

CONTACTS_PRIVILEDGE = "contacts"
VIEW_CONTACTS = "view_contacts"


class ContactsResource(Resource):
    """Resource class for contact items

    A contact can either be for an individual or for an organisation

    """

    schema = {
        # flag to mark the contact item active/inactive for example if the item relates to a contact that is no longer
        # valid
        "is_active": {"type": "boolean", "default": True},
        # flag to indicate that the contact item should not be publicly visible, for example if the item relates to
        # a contact that can not be distributed due to privacy concerns
        "public": {"type": "boolean", "default": True},
        "organisation": {
            "type": "string",
            "required": False,
            "mapping": text_with_keyword,
        },
        "first_name": {
            "type": "string",
            "required": False,
            "mapping": text_with_keyword,
        },
        "last_name": {
            "type": "string",
            "required": False,
            "mapping": text_with_keyword,
        },
        "honorific": {"type": "string", "required": False},
        "job_title": {"type": "string", "required": False},
        "mobile": {
            "type": "list",
            "schema": {
                "type": "dict",
                "schema": {
                    "number": {"type": "string"},
                    # usage may provide information as to the usage of the number e.g. Business hours only
                    "usage": {"type": "string"},
                    # If public is true then the number can be made public, if false it is only visible in the system
                    "public": {"type": "boolean"},
                },
            },
        },
        "contact_phone": {
            "type": "list",
            "schema": {
                "type": "dict",
                "schema": {
                    "number": {"type": "string"},
                    "usage": {"type": "string"},
                    # If public is true then the number can be made public, if false it is only visible in the system
                    "public": {"type": "boolean"},
                },
            },
        },
        "fax": {"type": "string", "required": False},
        "contact_email": {"type": "list", "required": False, "schema": {"type": "email"}},
        "twitter": {"type": "string", "required": False, "twitter": True},
        "facebook": {"type": "string", "required": False},
        "instagram": {"type": "string", "required": False},
        "website": {"type": "string", "required": False},
        "contact_address": {"type": "list", "schema": {"type": "string", "required": False}},
        "locality": {"type": "string", "required": False},
        "city": {"type": "string", "required": False},
        "contact_state": {"type": "string", "required": False},
        "postcode": {"type": "string", "required": False},
        "country": {
            "type": "dict",
            "schema": {
                "name": {"type": "string", "required": False},
                "qcode": {"type": "string", "required": False},
                "translations": {
                    "type": "dict",
                    "required": False,
                },
            },
        },
        "notes": {"type": "string", "required": False},
        "contact_type": {
            "type": "string",
            "required": False,
            "nullable": True,
        },
    }

    datasource = {"source": "contacts", "search_backend": "elastic"}
    item_url = r'regex("[\w]+")'
    privileges = {"POST": CONTACTS_PRIVILEDGE, "PATCH": CONTACTS_PRIVILEDGE, "DELETE": CONTACTS_PRIVILEDGE}


class OrganisationSearchResource(Resource):
    """Search for organisation within the contacts collection"""

    datasource = {"source": "contacts", "search_backend": "elastic"}

    schema = ContactsResource.schema
    resource_methods = ["GET"]
    item_methods = ["GET"]
    url = "contacts/organisations"
    resource_title = "contacts_organisations"



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from os.path import basename
from httmock import urlmatch, HTTMock
from urllib.parse import parse_qs


@urlmatch(scheme="https", netloc="commerce.reuters.com", path="/rmd/rest/xml/login")
def login_request(url, request):
    return {
        "status_code": 200,
        "content": '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><authToken>fake_token</authToken>',
    }


@urlmatch(scheme="http", netloc="rmb.reuters.com", path="/rmd/rest/xml/item")
def item_request(url, request):
    try:
        params = parse_qs(url.query, keep_blank_values=True)
        fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)), "../../tests/io/fixtures")
        if "channel" in params:
            file = os.path.join(fixtures, params["channel"][0])
        else:
            file = os.path.join(fixtures, params["id"][0].replace(":", "_version_"))
        with open(file, "r") as stored_response:
            content = stored_response.read()
            return {"status_code": 200, "content": content}
    except Exception:
        return {"status_code": 404}


@urlmatch(scheme="http", netloc="content.reuters.com")
def content_request(url, request):
    try:
        fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)), "../../tests/io/fixtures")
        file = os.path.join(fixtures, basename(url.path))
        with open(file, "rb") as stored_response:
            content = stored_response.read()
            return {"status_code": 200, "content": content}
    except Exception:
        return {"status_code": 404}


def setup_reuters_mock(context):
    context.mock = HTTMock(*[login_request, item_request, content_request])
    context.mock.__enter__()


def teardown_reuters_mock(context):
    if hasattr(context, "mock"):
        context.mock.__exit__(None, None, None)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from flask import current_app as app
from superdesk.services import BaseService
from superdesk.utils import ListCursor


logger = logging.getLogger(__name__)


class LanguagesService(BaseService):
    def get(self, req, lookup):
        """
        Return the list of languages defined on config file.
        """

        return ListCursor(app.config.get("LANGUAGES", []))



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : Jérôme
# Creation: 2019-11-06 18:08

from superdesk.commands.data_updates import BaseDataUpdate
from superdesk import get_resource_service
from apps.archive.common import transtype_metadata
from datetime import datetime


class DataUpdate(BaseDataUpdate):

    resource = "archive"

    def forwards(self, mongodb_collection, mongodb_database):
        vocabularies_service = get_resource_service("vocabularies")
        cursor = vocabularies_service.find({"field_type": "date"})
        if cursor.count() == 0:
            print('No field with "date" type, there is nothing to do')
        else:
            for resource in ["archive", "archive_autosave", "published"]:

                collection = mongodb_database[resource]

                for item in collection.find({"extra": {"$exists": True, "$ne": {}}}):
                    transtype_metadata(item)
                    print(
                        collection.update(
                            {"_id": item["_id"]},
                            {
                                "$set": {"extra": item["extra"]},
                            },
                        )
                    )

    def backwards(self, mongodb_collection, mongodb_database):
        vocabularies_service = get_resource_service("vocabularies")
        cursor = vocabularies_service.find({"field_type": "date"})
        if cursor.count() == 0:
            print('No field with "date" type, there is nothing to do')
        else:
            for resource in ["archive", "archive_autosave", "published"]:

                collection = mongodb_database[resource]

                for item in collection.find({"extra": {"$exists": True, "$ne": {}}}):
                    extra = item["extra"]
                    for key, value in extra.items():
                        if isinstance(value, datetime):
                            extra[key] = value.isoformat()

                    print(
                        collection.update(
                            {"_id": item["_id"]},
                            {
                                "$set": {"extra": extra},
                            },
                        )
                    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from apps.auth import get_user, get_user_id
from eve.versioning import resolve_document_version
from flask import request, current_app as app
from apps.archive import ArchiveSpikeService
from superdesk import get_resource_service, Service, config
from superdesk.metadata.item import (
    ITEM_STATE,
    EMBARGO,
    CONTENT_STATE,
    CONTENT_TYPE,
    ITEM_TYPE,
    PUBLISH_STATES,
    ASSOCIATIONS,
    GUID_TAG,
    PROCESSED_FROM,
    metadata_schema,
)
from superdesk.resource import Resource, build_custom_hateoas
from apps.archive.common import (
    CUSTOM_HATEOAS,
    ITEM_CREATE,
    ARCHIVE,
    BROADCAST_GENRE,
    ITEM_REWRITE,
    ITEM_UNLINK,
    ITEM_LINK,
    insert_into_versions,
)
from superdesk.metadata.utils import item_url, generate_guid
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.notification import push_notification
from superdesk.signals import item_rewrite
from apps.archive.archive import update_associations
from flask_babel import _

logger = logging.getLogger(__name__)


class ArchiveRewriteResource(Resource):
    endpoint_name = "archive_rewrite"
    resource_title = endpoint_name

    schema = metadata_schema.copy()
    schema.update(
        {
            "desk_id": {"type": "string", "nullable": True},
            "update": {"type": "dict", "nullable": True, "allow_unknown": True},
            "_links": {"type": "dict"},
        }
    )

    datasource = {
        "source": "archive",
        "search_backend": "elastic",
    }

    url = "archive/<{0}:original_id>/rewrite".format(item_url)
    resource_methods = ["POST", "DELETE"]
    privileges = {"POST": "rewrite", "DELETE": "rewrite"}


class ArchiveRewriteService(Service):
    def get(self, req, lookup):
        if lookup.get("original_id"):
            return super().get(req, {"_id": lookup["original_id"]})
        return super().get(req, lookup)

    def create(self, docs, **kwargs):
        doc = docs[0] if len(docs) > 0 else {}
        original_id = request.view_args["original_id"]
        update_document = doc.get("update")

        archive_service = get_resource_service(ARCHIVE)
        original = archive_service.find_one(req=None, _id=original_id)
        self._validate_rewrite(original, update_document)

        rewrite = self._create_rewrite_article(original, existing_item=update_document, desk_id=doc.get("desk_id"))

        if "body_html" in rewrite:
            if "fields_meta" in original:
                rewrite["fields_meta"] = original["fields_meta"]
            update_associations(rewrite)

        # signal
        item_rewrite.send(self, item=rewrite, original=original)

        if update_document:
            # process the existing story
            archive_service.patch(update_document[config.ID_FIELD], rewrite)
            app.on_archive_item_updated(rewrite, update_document, ITEM_LINK)
            rewrite[config.ID_FIELD] = update_document[config.ID_FIELD]
            ids = [update_document[config.ID_FIELD]]
        else:
            # Set the version.
            resolve_document_version(rewrite, ARCHIVE, "POST")
            ids = archive_service.post([rewrite])
            insert_into_versions(doc=rewrite)
            build_custom_hateoas(CUSTOM_HATEOAS, rewrite)

            app.on_archive_item_updated({"rewrite_of": rewrite.get("rewrite_of")}, rewrite, ITEM_LINK)

        self._add_rewritten_flag(original, rewrite)
        get_resource_service("archive_broadcast").on_broadcast_master_updated(
            ITEM_CREATE, item=original, rewrite_id=ids[0]
        )

        doc.clear()
        doc.update(rewrite)
        return ids

    def _validate_rewrite(self, original, update):
        """Validates the article to be rewritten.

        :param original: article to be rewritten
        :param update: article as the rewrite
        :raises: SuperdeskApiError
        """
        if not original:
            raise SuperdeskApiError.notFoundError(message=_("Cannot find the article"))

        if original.get(EMBARGO):
            raise SuperdeskApiError.badRequestError(_("Rewrite of an Item having embargo isn't possible"))

        if not original.get("event_id"):
            raise SuperdeskApiError.notFoundError(message=_("Event id does not exist"))

        if original.get("rewritten_by"):
            raise SuperdeskApiError.badRequestError(message=_("Article has been rewritten before !"))

        if (
            not is_workflow_state_transition_valid("rewrite", original[ITEM_STATE])
            and not config.ALLOW_UPDATING_SCHEDULED_ITEMS
        ):
            raise InvalidStateTransitionError()

        if (
            original.get("rewrite_of")
            and not (original.get(ITEM_STATE) in PUBLISH_STATES)
            and not app.config["WORKFLOW_ALLOW_MULTIPLE_UPDATES"]
        ):
            raise SuperdeskApiError.badRequestError(
                message=_("Rewrite is not published. Cannot rewrite the story again.")
            )

        if update:
            # in case of associate as update
            if update.get("rewrite_of"):
                raise SuperdeskApiError.badRequestError(_("Rewrite story has been used as update before !"))

            if update.get(ITEM_STATE) in [
                CONTENT_STATE.PUBLISHED,
                CONTENT_STATE.CORRECTED,
                CONTENT_STATE.KILLED,
                CONTENT_STATE.RECALLED,
                CONTENT_STATE.SCHEDULED,
                CONTENT_STATE.SPIKED,
            ]:
                raise InvalidStateTransitionError()

            if update.get(ITEM_TYPE) not in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
                raise SuperdeskApiError.badRequestError(_("Rewrite story can only be text or pre-formatted !"))

            if update.get("genre") and any(
                genre.get("value", "").lower() == BROADCAST_GENRE.lower() for genre in update.get("genre")
            ):
                raise SuperdeskApiError.badRequestError(_("Broadcast cannot be a update story !"))

            if original.get("profile") and original.get("profile") != update.get("profile"):
                raise SuperdeskApiError.badRequestError(
                    _("Rewrite item content profile does " "not match with Original item.")
                )

    def _create_rewrite_article(self, original, existing_item=None, desk_id=None):
        """Creates a new story and sets the metadata from original.

        :param dict original: original story
        :param dict existing_item: existing story that is being re-written
        :return:new story
        """
        rewrite = dict()

        fields = ["family_id", "event_id", "flags", "language", ASSOCIATIONS, "extra"]
        existing_item_preserve_fields = (ASSOCIATIONS, "flags")

        if app.config.get("COPY_ON_REWRITE_FIELDS"):
            fields.extend(app.config["COPY_ON_REWRITE_FIELDS"])

        if existing_item:
            # for associate an existing file as update merge subjects
            subjects = original.get("subject", [])
            unique_subjects = {subject.get("qcode") for subject in subjects}
            rewrite["subject"] = [
                subject for subject in existing_item.get("subject", []) if subject.get("qcode") not in unique_subjects
            ]
            rewrite["subject"].extend(subjects)
            rewrite["flags"] = original.get("flags") or {}

            # preserve flags
            for key in rewrite.get("flags").keys():
                rewrite["flags"][key] = original["flags"][key] or existing_item.get("flags", {}).get(key, False)

            original_associations = original.get(ASSOCIATIONS) or {}
            existing_associations = existing_item.get(ASSOCIATIONS) or {}
            rewrite[ASSOCIATIONS] = existing_associations

            # if the existing item has association then preserve the association
            for key, assoc in original_associations.items():
                if not existing_associations.get(key):
                    rewrite[ASSOCIATIONS][key] = assoc
        else:
            # ingest provider and source to be retained for new item
            fields.extend(["ingest_provider", "source"])

            if original.get("profile"):
                content_type = get_resource_service("content_types").find_one(req=None, _id=original["profile"])
                extended_fields = list(content_type["schema"].keys())
                # extra fields needed.
                extended_fields.extend(["profile", "keywords", "target_regions", "target_types", "target_subscribers"])
            else:
                extended_fields = [
                    "abstract",
                    "anpa_category",
                    "pubstatus",
                    "slugline",
                    "urgency",
                    "subject",
                    "priority",
                    "byline",
                    "dateline",
                    "headline",
                    "place",
                    "genre",
                    "body_footer",
                    "company_codes",
                    "keywords",
                    "target_regions",
                    "target_types",
                    "target_subscribers",
                ]

            fields.extend(extended_fields)

        for field in fields:
            if original.get(field):
                # don't overwrite some fields in existing items
                if existing_item and field in existing_item_preserve_fields:
                    continue

                rewrite[field] = original[field]

        # if the original was flagged for SMS the rewrite should not be.
        if not existing_item and rewrite.get("flags", {}).get("marked_for_sms", False):
            rewrite["flags"]["marked_for_sms"] = False

        # SD-4595 - Default value for the update article to be set based on the system config.
        if config.RESET_PRIORITY_VALUE_FOR_UPDATE_ARTICLES:
            # if True then reset to the default priority value.
            rewrite["priority"] = int(config.DEFAULT_PRIORITY_VALUE_FOR_MANUAL_ARTICLES)

        rewrite["rewrite_of"] = original[config.ID_FIELD]
        rewrite["rewrite_sequence"] = (original.get("rewrite_sequence") or 0) + 1
        rewrite.pop(PROCESSED_FROM, None)

        if not existing_item:
            from apps.tasks import send_to

            # send the document to the desk only if a new rewrite is created
            send_to(
                doc=rewrite,
                desk_id=(desk_id or original["task"]["desk"]),
                default_stage="working_stage",
                user_id=get_user_id(),
            )

            # if we are rewriting a published item then copy the body_html
            if original.get("state", "") in (CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED, CONTENT_STATE.SCHEDULED):
                rewrite["body_html"] = original.get("body_html", "")

        rewrite[ITEM_STATE] = CONTENT_STATE.PROGRESS
        self._set_take_key(rewrite)
        return rewrite

    def _add_rewritten_flag(self, original, rewrite):
        """Adds rewritten_by field to the existing published items.

        :param dict original: item on which rewrite is triggered
        :param dict rewrite: rewritten document
        """
        get_resource_service("published").update_published_items(
            original[config.ID_FIELD], "rewritten_by", rewrite[config.ID_FIELD]
        )

        # modify the original item as well.
        get_resource_service(ARCHIVE).system_update(
            original[config.ID_FIELD], {"rewritten_by": rewrite[config.ID_FIELD]}, original
        )
        app.on_archive_item_updated({"rewritten_by": rewrite[config.ID_FIELD]}, original, ITEM_REWRITE)

    def _set_take_key(self, rewrite):
        """Sets the anpa take key of the rewrite with ordinal.

        :param rewrite: rewrite story
        """
        rewrite_sequence = rewrite.get("rewrite_sequence") or 0
        if rewrite_sequence > 1:
            ordinal = self._get_ordinal(rewrite_sequence)
            rewrite["anpa_take_key"] = "{} update".format(ordinal)
        else:
            rewrite["anpa_take_key"] = "update"

    def _get_ordinal(self, n):
        """Returns the ordinal value of the given int."""
        if 10 <= n % 100 < 20:
            return str(n) + "th"
        else:
            return str(n) + {1: "st", 2: "nd", 3: "rd"}.get(n % 10, "th")

    def delete(self, lookup):
        target_id = request.view_args["original_id"]
        archive_service = get_resource_service(ARCHIVE)
        target = archive_service.find_one(req=None, _id=target_id)
        updates = {}

        if target.get("rewrite_of"):
            # remove the rewrite info
            ArchiveSpikeService().update_rewrite(target)

        if not target.get("rewrite_of"):
            # there is nothing to do
            raise SuperdeskApiError.badRequestError(_("Only updates can be unlinked!"))

        if target.get("rewrite_of"):
            updates["rewrite_of"] = None

        if target.get("anpa_take_key"):
            updates["anpa_take_key"] = None

        if target.get("rewrite_sequence"):
            updates["rewrite_sequence"] = None

        if target.get("sequence"):
            updates["sequence"] = None

        updates["event_id"] = generate_guid(type=GUID_TAG)

        archive_service.system_update(target_id, updates, target)
        user = get_user(required=True)
        push_notification("item:unlink", item=target_id, user=str(user.get(config.ID_FIELD)))
        app.on_archive_item_updated(updates, target, ITEM_UNLINK)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from eve.utils import ParsedRequest
from superdesk import get_resource_service
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from apps.publish.enqueue import get_enqueue_service
from apps.archive.common import ITEM_OPERATION
from apps.publish.content.common import BasePublishService
from flask_babel import _

logger = logging.getLogger(__name__)


class ProductTestResource(Resource):
    endpoint_name = "product_tests"
    schema = {"article_id": {"type": "string"}}
    datasource = {
        "projection": {
            "_items": 1,
        },
    }
    url = "products/test"
    resource_methods = ["POST"]
    item_methods = []
    resource_title = endpoint_name
    privileges = {"POST": "products"}


class ProductTestService(BaseService):
    def create(self, docs, **kwargs):
        archive_service = get_resource_service("archive")
        doc = docs[0]
        if not doc.get("article_id"):
            raise SuperdeskApiError.badRequestError(_("Article id cannot be empty!"))

        article_id = doc.get("article_id")
        article = archive_service.find_one(req=None, _id=article_id)

        if not article:
            raise SuperdeskApiError.badRequestError(_("Article not found!"))

        try:
            results = self.test_products(article)
        except Exception as ex:
            logger.exception(ex)
            raise SuperdeskApiError.badRequestError(
                _("Error in testing article: {exception}").format(exception=str(ex))
            )

        doc["_items"] = results
        return [article_id]

    def test_products(self, article, lookup=None):
        req = ParsedRequest()
        results = []
        products = list(get_resource_service("products").get(req=req, lookup=lookup))
        service = get_enqueue_service(article.get(ITEM_OPERATION, "publish"))
        for product in products:
            result = {"product_id": product["_id"], "matched": True, "name": product.get("name", "")}
            reason = ""
            if not service.conforms_product_targets(product, article):
                # Here it fails to match due to geo restriction
                # story has target_region and product has geo restriction
                result["matched"] = False

                if BasePublishService().is_targeted(article, "target_regions"):
                    reason = "Story has target_region"

                if product.get("geo_restrictions"):
                    reason = "{} {}".format(reason, "Product has target_region")

            if not service.conforms_content_filter(product, article):
                # Here it fails to match due to content filter
                content_filter = product.get("content_filter")
                filter = service.filters.get("content_filters", {}).get(content_filter["filter_id"], {}).get("cf")
                result["matched"] = False
                reason = "Story does not match the filter: {}".format(filter.get("name"))

            result["reason"] = reason
            results.append(result)
        return results



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json

from eve.utils import config, ParsedRequest
from flask import request, current_app as app

import superdesk
from apps.archive.archive import SOURCE as ARCHIVE
from apps.auth import get_user, get_user_id
from apps.content import push_content_notification
from apps.tasks import send_to
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE
from superdesk.metadata.utils import item_url
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk.utc import utcnow
from flask_babel import _


class DuplicateResource(Resource):
    endpoint_name = "duplicate"
    resource_title = endpoint_name

    schema = {
        "desk": Resource.rel("desks", False, required=True, nullable=True),
        "stage": Resource.rel("stages", False, required=False),
        "type": {"type": "string", "required": True},
        "item_id": {"type": "string", "required": False},
    }

    url = "archive/<{0}:guid>/duplicate".format(item_url)

    resource_methods = ["POST"]
    item_methods = []

    privileges = {"POST": "duplicate"}


class DuplicateService(BaseService):
    def on_create(self, docs):
        for doc in docs:
            if not doc.get("desk") and not app.config["WORKFLOW_ALLOW_COPY_TO_PERSONAL"]:
                raise SuperdeskApiError.forbiddenError(message=_("Duplicate to Personal space is not allowed."))

    def create(self, docs, **kwargs):
        guid_of_item_to_be_duplicated = request.view_args["guid"]

        guid_of_duplicated_items = []

        for doc in docs:
            archive_service = get_resource_service(ARCHIVE)
            archived_doc = {}

            if doc.get("type") == "archived":
                archived_service = get_resource_service("archived")
                req = ParsedRequest()
                query = {
                    "query": {"filtered": {"filter": {"bool": {"must": [{"term": {"item_id": doc.get("item_id")}}]}}}},
                    "sort": [{"_current_version": "desc"}],
                    "size": 1,
                }
                req.args = {"source": json.dumps(query)}
                archived_docs = archived_service.get(req=req, lookup=None)
                if archived_docs.count() > 0:
                    archived_doc = archived_docs[0]

            else:
                archived_doc = archive_service.find_one(req=None, _id=guid_of_item_to_be_duplicated)

            self._validate(archived_doc, doc, guid_of_item_to_be_duplicated)

            # reset timestamps
            archived_doc["versioncreated"] = archived_doc["firstcreated"] = utcnow()
            archived_doc["firstpublished"] = None

            send_to(
                doc=archived_doc,
                desk_id=doc.get("desk"),
                stage_id=doc.get("stage"),
                default_stage="working_stage",
                user_id=get_user_id(),
            )

            if not doc.get("desk"):  # item copied to personal space
                archived_doc["state"] = CONTENT_STATE.PROGRESS

            new_guid = archive_service.duplicate_content(archived_doc)
            guid_of_duplicated_items.append(new_guid)

        if kwargs.get("notify", True):
            push_content_notification([archived_doc])

        return guid_of_duplicated_items

    def _validate(self, doc_in_archive, doc, guid_to_duplicate):
        """Validates if the given archived_doc is still eligible to be duplicated.

        Rules:
            1. Is the item requested found in archive collection?
            2. Is workflow transition valid?
            3. Is item locked by another user?

        :param doc_in_archive: object representing the doc in archive collection
        :type doc_in_archive: dict
        :param doc: object received as part of request
        :type doc: dict
        :param guid_to_duplicate: GUID of the item to duplicate
        :type guid_to_duplicate: str
        :raises
            SuperdeskApiError.notFoundError: If doc_in_archive is None
            SuperdeskApiError.forbiddenError: if item is locked
            InvalidStateTransitionError: if workflow transition is invalid
        """

        if not doc_in_archive:
            raise SuperdeskApiError.notFoundError(
                _("Fail to found item with guid: {guid}").format(guid=guid_to_duplicate)
            )

        if not is_workflow_state_transition_valid("duplicate", doc_in_archive[ITEM_STATE]):
            raise InvalidStateTransitionError()

        lock_user = doc_in_archive.get("lock_user", None)
        force_unlock = doc_in_archive.get("force_unlock", False)
        user = get_user()
        str_user_id = str(user.get(config.ID_FIELD)) if user else None
        if lock_user and str(lock_user) != str_user_id and not force_unlock:
            raise SuperdeskApiError.forbiddenError(_("The item was locked by another user"))


superdesk.workflow_action(
    name="duplicate",
    exclude_states=[CONTENT_STATE.SPIKED, CONTENT_STATE.KILLED, CONTENT_STATE.RECALLED],
    privileges=["archive", "duplicate"],
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from copy import deepcopy

from superdesk.tests import TestCase
from superdesk.io.commands.update_ingest import process_iptc_codes


class IPTCCodesTestCase(TestCase):
    def test_unknown_iptc(self):
        """Test if an unknown IPTC code is not causing a crash"""
        item = {
            "guid": "urn:newsml:localhost:2019-02-07T12:00:00.030513:369c16e0-d6b7-40e1-8838-9c5f6a61626c",
            "subject": [{"name": "system", "qcode": "99009000"}],
        }
        # item should not be modified
        expected = deepcopy(item)

        with self.app.app_context():
            process_iptc_codes(item, {})
        self.assertEqual(item, expected)



import superdesk

from superdesk.notification import push_notification
from apps.content import push_content_notification
from superdesk import get_resource_service

PRIVILEGE = "desk_routing"


class ClosedDeskResource(superdesk.Resource):
    """Custom resource to change desk.is_closed

    Allowing users without desk management privileges to close/open desk.
    """

    schema = {
        "is_closed": {"type": "boolean"},
        "closed_destination": superdesk.Resource.rel("desks"),
    }

    datasource = {
        "source": "desks",
    }

    privileges = {
        "PATCH": PRIVILEGE,  # anyone who can edit can turn it on/off
    }

    resource_methods = []
    item_methods = ["PATCH"]


class ClosedDeskService(superdesk.Service):
    def on_updated(self, updates, original):
        is_closed = updates.get("is_closed", False)
        if not is_closed and original.get("is_closed"):
            desk_id = updates["_id"]
            self.remove_marks(desk_id)
        push_notification("desks:closed", is_closed=is_closed, _id=original.get("_id"), _etag=updates.get("_etag"))

    def remove_marks(self, desk_id):
        """Remove "mark for desk" attribute

        :param ObjectId desk_id: id of the desk being re-opened
        """
        for service_name in ("archive", "published"):
            service = get_resource_service(service_name)
            marked_items = service.find({"task.desk": desk_id, "marked_desks": {"$exists": True, "$ne": []}})
            for item in marked_items:
                marked_desks = [m for m in item["marked_desks"] if "user_marked" in m]
                service.system_update(item["_id"], {"marked_desks": marked_desks}, item)
                push_content_notification([item])


def init_app(app):
    app.config["DOMAIN"]["desks"]["schema"].update(
        {
            "is_closed": {"type": "boolean"},
            "closed_destination": superdesk.Resource.rel("desks"),
        }
    )

    app.config["SOURCES"]["desks"]["projection"].update(
        {
            "is_closed": 1,
            "closed_destination": 1,
        }
    )

    superdesk.register_resource("closed_desks", ClosedDeskResource, ClosedDeskService, _app=app)

    superdesk.privilege(name=PRIVILEGE, label="Desk routing", description="User can configure desk routing")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging

from superdesk.io.registry import register_feeding_service
from superdesk.io.feeding_services.http_base_service import HTTPFeedingServiceBase
from superdesk.errors import IngestApiError, SuperdeskIngestError
from lxml import etree

logger = logging.getLogger(__name__)
NS = {"iptc": "http://iptc.org/std/nar/2006-10-01/"}


class APFeedingService(HTTPFeedingServiceBase):
    """
    Feeding Service class which can retrieve articles from Associated Press web service
    """

    NAME = "ap"

    label = "AP feed API"

    fields = HTTPFeedingServiceBase.AUTH_FIELDS + [
        {
            "id": "idList",
            "type": "text",
            "label": "Id List",
            "placeholder": "use coma separated ids for multiple values",
            "required": False,
        },
        {
            "id": "idListType",
            "type": "choices",
            "label": "Id List Type",
            "choices": (
                ("products", "Products"),
                ("savedsearches", "Saved searches"),
                ("topics", "Topics"),
                ("offerings", "Offerings"),
                ("packages", "Packages"),
            ),
            "default": "products",
            "required": True,
        },
    ]
    HTTP_URL = "https://syndication.ap.org/AP.Distro.Feed/GetFeed.aspx"

    def config_test(self, provider=None):
        super().config_test(provider)

    def _update(self, provider, update):
        try:
            config = provider["config"]
            id_list = config["idList"]
            # before "products" was hardcoded as value for "idListType"
            id_list_type = config.get("idListType", "products")
            if not id_list.strip():
                raise KeyError
        except KeyError:
            raise SuperdeskIngestError.notConfiguredError(Exception("idList is needed"))

        # we check if the provider has been closed since the last update
        try:
            last_closed = provider["last_closed"]["closed_at"]
            last_updated = provider["last_updated"]
        except KeyError:
            pass
        else:
            if last_closed > last_updated and "private" in provider:
                # we reset the private data so only last page of items will be retrieved (cf. SDESK-4372)
                logger.info("reseting private data for provider {source}".format(source=provider.get("source")))
                del provider["private"]

        # we remove spaces and empty values from id_list to do a clean list
        id_list = ",".join([id_.strip() for id_ in id_list.split(",") if id_.strip()])

        params = {
            "idList": id_list,
            "idListType": id_list_type,
            "format": "5",
            "maxItems": "25",
        }
        try:
            min_date_time = provider["private"]["min_date_time"]
            sequence_number = provider["private"]["sequence_number"]
        except KeyError:
            # the provider is new or re-opened, we want last items
            # so we need reverse-chronological order
            chronological = False
        else:
            params["minDateTime"] = min_date_time
            params["sequenceNumber"] = sequence_number
            params["sortOrder"] = "chronological"
            chronological = True

        r = self.get_url(params=params)

        try:
            root_elt = etree.fromstring(r.content)
        except Exception:
            raise IngestApiError.apiRequestError(Exception("error while doing the request"))

        parser = self.get_feed_parser(provider)
        items = parser.parse(root_elt, provider)
        if not chronological:
            items.reverse()

        try:
            min_date_time = root_elt.xpath('//iptc:timestamp[@role="minDateTime"]/text()', namespaces=NS)[0].strip()
            sequence_number = root_elt.xpath("//iptc:transmitId/text()", namespaces=NS)[0].strip()
        except IndexError:
            raise IngestApiError.apiRequestError(Exception("missing minDateTime or transmitId"))
        else:
            update.setdefault("private", {})
            update["private"]["min_date_time"] = min_date_time
            update["private"]["sequence_number"] = sequence_number

        return [items]


register_feeding_service(APFeedingService)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from lxml import etree as etree
from lxml.etree import SubElement
from flask import current_app as app
from superdesk.publish.formatters import Formatter
from superdesk.errors import FormatterError
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, EMBARGO, FORMAT, FORMATS
from apps.archive.common import get_utc_schedule
from superdesk.text_utils import get_text


class EraseElement(Exception):
    """Remove entirely element"""


class NITFFormatter(Formatter):
    """NITF Formatter for Superdesk

    Format items to `NITF <https://iptc.org/standards/nitf/>`_ version *3.6*.
    """

    XML_ROOT = '<?xml version="1.0"?>'
    ENCODING = "UTF-8"

    _message_attrib = {"version": "-//IPTC//DTD NITF 3.6//EN"}

    _schema_uri = "http://www.iptc.org/std/NITF/3.6/specification"
    _schema_ref = "http://www.iptc.org/std/NITF/3.6/specification/nitf-3-6.xsd"
    _debug_message_extra = {"schemaLocation": "{} {}".format(_schema_uri, _schema_ref)}
    NITF_COMMON_ATTR = ("id", "class", "style")
    NITF_CELL_ALIGN = ("align", "char", "charoff")
    NITF_CELL_VALIGN = ("valign",)
    NITF_ALLOWED_ATTR = {
        "p": NITF_COMMON_ATTR + ("lede", "summary", "optional-tex"),
        "a": NITF_COMMON_ATTR + ("href", "name", "rel", "rev", "title"),
        "br": ("id",),
        "em": NITF_COMMON_ATTR,
        "q": NITF_COMMON_ATTR,
        "hl1": NITF_COMMON_ATTR,
        "hl2": NITF_COMMON_ATTR,
        "table": NITF_COMMON_ATTR
        + ("tabletype", "align", "width", "cols", "border", "frame", "rules", "cellspacing", "cellpadding"),
        "tbody": NITF_COMMON_ATTR + NITF_CELL_ALIGN + NITF_CELL_VALIGN,
        "tr": NITF_COMMON_ATTR + NITF_CELL_ALIGN + NITF_CELL_VALIGN,
        "th": NITF_COMMON_ATTR + NITF_CELL_ALIGN + NITF_CELL_VALIGN + ("axis", "axes", "nowrap", "rowspan", "colspan"),
        "td": NITF_COMMON_ATTR + NITF_CELL_ALIGN + NITF_CELL_VALIGN + ("axis", "axes", "nowrap", "rowspan", "colspan"),
        "nitf-table": ("id",),
        "ol": NITF_COMMON_ATTR + ("seqnum",),
        "ul": NITF_COMMON_ATTR,
        "li": NITF_COMMON_ATTR,
        "dl": NITF_COMMON_ATTR,
        "dt": NITF_COMMON_ATTR,
        "dd": NITF_COMMON_ATTR,
        "bq": NITF_COMMON_ATTR + ("nowrap", "quote-source"),
        "fn": NITF_COMMON_ATTR,
        "note": NITF_COMMON_ATTR + ("noteclass", "type"),
        "pre": NITF_COMMON_ATTR,
        "sup": NITF_COMMON_ATTR,
        "sub": NITF_COMMON_ATTR,
        "hr": NITF_COMMON_ATTR,
    }

    NITF_ENRICHED_TEXT_PARENTS = (
        "a",
        "caption",
        "credit",
        "dd",
        "dt",
        "em",
        "hl1",
        "hl2",
        "lang",
        "li",
        "media-caption",
        "media-producer",
        "p",
        "pronounce",
        "q",
        "tagline",
        "td",
        "th",
    )

    def __init__(self):
        Formatter.__init__(self)

        # in elements' dicts, following key can be used:
        # - 'nitf': new NITF compatible element to use (empty string to remove)
        # - 'attrib': new attribute to use (replace existing one if set)
        # - 'filter': callback to use for complex changes (root_elment and element as arguments)
        self.HTML2NITF = {
            "p": {},
            "b": {"nitf": "em", "attrib": {"class": "bold"}},
            "br": {"filter": self.br_filter},
            "strong": {"nitf": "em", "attrib": {"class": "bold"}},
            "i": {"nitf": "em", "attrib": {"class": "italic"}},
            "em": {"nitf": "em", "attrib": {"class": "italic"}},
            "u": {"nitf": "em", "attrib": {"class": "underscore"}},
            "strike": {"nitf": "em"},
            "sup": {},
            "sub": {},
            "a": {},
            "img": {"nitf": ""},  # <img> use <media> in nitf, so we remove element
            "blockquote": {"nitf": "bq"},
            "pre": {},
            "ol": {},
            "ul": {},
            "li": {},
            # FIXME: hl1 is not used here as it can only appear in <hedline>
            "h1": {"nitf": "hl2"},
            "h2": {"nitf": "hl2"},
            "h3": {"nitf": "hl2"},
            "h4": {"nitf": "hl2"},
            "h5": {"nitf": "hl2"},
            "h6": {"nitf": "hl2"},
            # tables
            "table": {},
            "tbody": {},
            "tr": {},
            "td": {},
            "th": {},
            "style": {"nitf": EraseElement},  # <style> may be there in case of bad paste
        }

    def format(self, article, subscriber, codes=None):
        try:
            pub_seq_num = superdesk.get_resource_service("subscribers").generate_sequence_number(subscriber)

            nitf = self.get_nitf(article, subscriber, pub_seq_num)
            return [
                (
                    pub_seq_num,
                    self.XML_ROOT
                    + etree.tostring(nitf, pretty_print=True, encoding=self.ENCODING).decode(self.ENCODING),
                )
            ]
        except Exception as ex:
            raise FormatterError.nitfFormatterError(ex, subscriber)

    def get_nitf(self, article, destination, pub_seq_num):
        if app.config.get("NITF_INCLUDE_SCHEMA", False):
            self._message_attrib.update(self._debug_message_extra)
        nitf = etree.Element("nitf", attrib=self._message_attrib)
        head = SubElement(nitf, "head")
        body = SubElement(nitf, "body")
        body_head = SubElement(body, "body.head")
        body_content = SubElement(body, "body.content")

        body_end = SubElement(body, "body.end")

        self._format_title(article, head)
        self._format_meta(article, head, destination, pub_seq_num)
        self._format_head(article, head)
        self._format_body_head(article, body_head)
        self._format_body_content(article, body_content)
        self._format_body_end(article, body_end)
        return nitf

    def _textToParent(self, parent, children, idx, text):
        """Copy Element.text to parent or sibling element

        for internal use only
        """
        # we copy text to sibling element's tail
        # or to parent text if it is the first element
        if idx > 0:
            sibling = children[idx - 1]
            sibling.tail = (sibling.tail or "") + text
        else:
            parent.text = (parent.text or "") + text

    def _parse_children(self, html_elem, root_elem, attr_remove):
        children = list(html_elem)
        idx = 0
        while idx < len(children):
            child = children[idx]
            try:
                self.html2nitf(child, root_elem=root_elem, attr_remove=attr_remove)
            except ValueError:
                # the element is unknown
                # we need to save its text and tail,
                # and put its children to parent
                grandchildren = list(child)

                if child.text:
                    self._textToParent(html_elem, children, idx, child.text)

                if child.tail:
                    # we copy tail to last grandchild tail
                    # or we append to parent/sibling if there is no grandchild
                    if grandchildren:
                        last = grandchildren[-1]
                        last.tail = (last.tail or "") + child.tail
                    else:
                        self._textToParent(html_elem, children, idx, child.tail)

                # we move elem children to parent
                for grandchild_idx, grandchild in enumerate(grandchildren):
                    insert_idx = idx + grandchild_idx
                    html_elem.insert(insert_idx, grandchild)
                    children.insert(insert_idx, grandchild)

                # and remove the element
                html_elem.remove(child)
                children.remove(child)

                # and we continue with the same index, so new children are parsed
                continue
            except EraseElement:
                # the element need to be fully erased (itself and its children)
                html_elem.remove(child)
                children.remove(child)
                continue
            idx += 1

        return html_elem

    def html2nitf(self, html_elem, root_elem=None, attr_remove=None):
        """Convert HTML elements to NITF compatible elements

        :param ET.Element: HTML to clean/transform
        :param ET.Element: None if its the root element (must be a <div>)
        :param list attr_remove: attributes to remove if present
        :return ET.Element: <div> element with NITF compliant children
        """
        if root_elem is None:
            root_elem = html_elem
            assert html_elem.tag == "div"
            # we change children of root element in place
            return self._parse_children(html_elem, root_elem, attr_remove)

        try:
            nitf_map = self.HTML2NITF[html_elem.tag]
        except KeyError:
            raise ValueError("Unhandled HTML element")
        nitf_elem = nitf_map.get("nitf")
        if nitf_elem is not None:
            if nitf_elem == "":
                raise ValueError("Element need to be removed")
            elif nitf_elem == EraseElement:
                raise EraseElement("Element need to be erased")
            html_elem.tag = nitf_elem

        if attr_remove is not None:
            for attr in attr_remove:
                if attr in html_elem.attrib:
                    del html_elem.attrib[attr]

        html_elem.attrib.update(nitf_map.get("attrib", {}))

        if "filter" in nitf_map:
            nitf_map["filter"](root_elem, html_elem)

        attr_allowed = self.NITF_ALLOWED_ATTR.get(html_elem.tag, ())

        for attr in list(html_elem.attrib):
            if attr not in attr_allowed:
                del html_elem.attrib[attr]

        return self._parse_children(html_elem, root_elem, attr_remove)

    def br_filter(self, root_elem, html_elem):
        parent = html_elem.find("..")
        if parent.tag not in self.NITF_ENRICHED_TEXT_PARENTS:
            raise ValueError("this element need to be removed")
        elif parent.tag == "p":
            if parent[-1] == html_elem and not (html_elem.tail or "").strip():
                raise ValueError("this element need to be removed")

    def _format_tobject(self, article, head):
        return SubElement(head, "tobject", {"tobject.type": "news"})

    def _format_docdata_dateissue(self, article, docdata):
        SubElement(docdata, "date.issue", {"norm": str(article.get("firstcreated", ""))})

    def _format_docdata_doc_id(self, article, docdata):
        SubElement(docdata, "doc-id", attrib={"id-string": article.get("guid", "")})

    def _format_docdata(self, article, docdata):
        SubElement(docdata, "urgency", {"ed-urg": str(article.get("urgency", ""))})
        self._format_docdata_dateissue(article, docdata)
        self._format_docdata_doc_id(article, docdata)

        # ednote can exist and be set to None, that why we can't use default
        # value for article.get
        SubElement(docdata, "ed-msg", {"info": article.get("ednote") or ""})

    def _format_pubdata(self, article, head):
        pass

    def _format_title(self, article, head):
        title = SubElement(head, "title")
        title.text = article.get("headline", "")

    def _format_date_expire(self, article, docdata):
        if article.get(EMBARGO):
            docdata.attrib["management-status"] = "embargoed"
            SubElement(docdata, "date.expire", {"norm": str(get_utc_schedule(article, EMBARGO).isoformat())})
        else:
            docdata.attrib["management-status"] = article.get("pubstatus", "")
            SubElement(docdata, "date.expire", {"norm": str(article.get("expiry", ""))})

    def _format_head(self, article, head):
        tobject = self._format_tobject(article, head)
        if article.get("genre"):
            SubElement(tobject, "tobject.property", {"tobject.property.type": article["genre"][0]["name"]})
        self._format_subjects(article, tobject)

        docdata = SubElement(head, "docdata")
        self._format_date_expire(article, docdata)
        self._format_docdata(article, docdata)
        self._format_pubdata(article, head)
        self._format_keywords(article, head)

    def _format_subjects(self, article, tobject):
        for subject in article.get("subject", []):
            SubElement(tobject, "tobject.subject", {"tobject.subject.refnum": subject.get("qcode", "")})

    def _format_keywords(self, article, head):
        if article.get("keywords"):
            keylist = SubElement(head, "key-list")
            for keyword in article["keywords"]:
                SubElement(keylist, "keyword", {"key": keyword})

    def _format_body_head_abstract(self, article, body_head):
        if article.get("abstract"):
            abstract = SubElement(body_head, "abstract")
            self.map_html_to_xml(abstract, article.get("abstract"))

    def _format_body_head_distributor(self, article, body_head):
        pass

    def _format_body_head_dateline(self, article, body_head):
        if article.get("dateline", {}).get("text"):
            dateline = SubElement(body_head, "dateline")
            dateline.text = article["dateline"]["text"]

    def _format_body_head(self, article, body_head):
        hedline = SubElement(body_head, "hedline")
        hl1 = SubElement(hedline, "hl1")
        hl1.text = article.get("headline", "")

        if article.get("byline"):
            byline = SubElement(body_head, "byline")
            byline.text = article["byline"]

        self._format_body_head_distributor(article, body_head)
        self._format_body_head_dateline(article, body_head)
        self._format_body_head_abstract(article, body_head)

        for company in article.get("company_codes", []):
            org = SubElement(
                body_head,
                "org",
                attrib={"idsrc": company.get("security_exchange", ""), "value": company.get("qcode", "")},
            )
            org.text = company.get("name", "")

    def _format_body_content(self, article, body_content):
        if article.get(FORMAT) == FORMATS.PRESERVED:
            pre = get_text(self.append_body_footer(article))
            SubElement(body_content, "pre").text = pre
        else:
            self.map_html_to_xml(body_content, self.append_body_footer(article))

    def _format_body_end(self, article, body_end):
        if article.get("ednote"):
            tagline = SubElement(body_end, "tagline")
            tagline.text = article["ednote"]

    def can_format(self, format_type, article):
        return format_type == "nitf" and article[ITEM_TYPE] in (
            CONTENT_TYPE.TEXT,
            CONTENT_TYPE.PREFORMATTED,
            CONTENT_TYPE.COMPOSITE,
        )

    def _format_meta_priority(self, article, head):
        pass

    def _format_meta(self, article, head, destination, pub_seq_num):
        """
        Appends <meta> elements to <head>
        """
        self._format_meta_priority(article, head)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk

from superdesk.errors import SuperdeskApiError
from superdesk.utils import ListCursor
from .macro_register import macros


def get_public_props(item):
    return {k: v for k, v in item.items() if k != "callback"}


class MacrosService(superdesk.Service):
    def get(self, req, lookup):
        """Return all registered macros."""
        desk = getattr(req, "args", {}).get("desk")
        include_backend = getattr(req, "args", {}).get("backend") == "true"
        all_macros = self.get_macros(include_backend)

        if desk:
            return ListCursor(
                [
                    get_public_props(macro)
                    for macro in all_macros
                    if desk.upper() in macro.get("desks", []) or macro.get("desks") is None
                ]
            )
        else:
            return ListCursor([get_public_props(macro) for macro in all_macros])

    def create(self, docs, **kwargs):
        try:
            ids = []
            for doc in docs:
                res = self.execute_macro(doc["item"], doc["macro"])
                if isinstance(res, tuple):
                    doc["item"] = res[0]
                    doc["diff"] = res[1]
                else:
                    doc["item"] = res
                if doc.get("commit"):
                    item = superdesk.get_resource_service("archive").find_one(req=None, _id=doc["item"]["_id"])
                    updates = doc["item"].copy()
                    updates.pop("_id")
                    superdesk.get_resource_service("archive").update(item["_id"], updates, item)
                ids.append(doc["macro"])
            return ids
        except Exception as ex:
            raise SuperdeskApiError.internalError(str(ex), exception=ex)

    def get_macro_by_name(self, macro_name):
        return macros.find(macro_name)

    def execute_macro(self, doc, macro_name, **kwargs):
        macro = self.get_macro_by_name(macro_name)
        return macro["callback"](doc, **kwargs)

    def get_macros(self, include_backend):
        if include_backend:
            return macros
        else:
            return [m for m in macros if m.get("access_type") == "frontend"]

    def execute_translation_macro(self, doc, from_language, to_language):
        """
        Apply to doc all macros that are related to current translation defined by from_language and to_language
        Macros can have optionally defined translation related settings: from_languages, to_languages.
        """
        for m in macros:
            if (not m.get("from_languages", None) or from_language in m["from_languages"]) and to_language in m.get(
                "to_languages", []
            ):
                m["callback"](doc, from_language=from_language, to_language=to_language)


class MacrosResource(superdesk.Resource):
    resource_methods = ["GET", "POST"]
    item_methods = []
    privileges = {"POST": "archive"}

    schema = {
        "macro": {"type": "string", "required": True, "allowed": macros},
        "item": {
            "type": "dict",
        },
        "commit": {
            "type": "boolean",
            "default": False,
        },
        "diff": {
            "type": "dict",
            "readonly": True,
        },
        "name": {
            "type": "string",
            "readonly": True,
        },
        "label": {
            "type": "string",
            "readonly": True,
        },
        "description": {
            "type": "string",
            "readonly": True,
        },
        "action_type": {
            "type": "string",
            "readonly": True,
        },
        "access_type": {
            "type": "string",
            "readonly": True,
        },
        "group": {
            "type": "string",
            "readonly": True,
        },
    }



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : tomas
# Creation: 2018-11-27 10:54

from superdesk.commands.data_updates import BaseDataUpdate
from superdesk import get_resource_service

# This upgrade script does the same as the previous one 00014_20181114-153727_archive.py
# except this works across multiple collections


def get_root_nodes(tree_items):
    root_nodes = []

    for key in tree_items:
        node = tree_items[key]
        if node.parent is None:
            root_nodes.append(node)

    return root_nodes


def get_ids_recursive(list_of_nodes, resource):
    # walks the tree and returns ids of items with a specified resource

    ids = []

    for node in list_of_nodes:
        if len(node.children) > 0:
            ids.extend(get_ids_recursive(node.children, resource))

        if node.resource == resource:
            ids.append(node.id)

    return ids


class TreeNode:
    def __init__(self, id):
        self.id = id
        self.parent = None
        self.resource = None
        self.children = []


class DataUpdate(BaseDataUpdate):

    resource = "archive"  # will use multiple resources, keeping this here so validation passes

    def forwards(self, mongodb_collection, mongodb_database):
        tree_items = {}

        # `translated_from` can refer to archive['_id'] or published['item_id']

        for resource in ["archive", "published"]:
            collection = mongodb_database[resource]

            # building multiple trees
            for item in collection.find({"translated_from": {"$exists": True}}):
                node_id = item["_id"]

                if node_id not in tree_items:
                    tree_items[node_id] = TreeNode(node_id)

                node = tree_items[node_id]
                node.resource = resource
                parent_id = item["translated_from"]

                if parent_id not in tree_items:
                    tree_items[parent_id] = TreeNode(parent_id)

                node.parent = tree_items[parent_id]
                node.parent.children.append(node)

        # processing trees
        for root_node in get_root_nodes(tree_items):
            updates = {"translation_id": root_node.id}

            for resource in ["archive", "published"]:
                service = get_resource_service(resource)
                ids = get_ids_recursive([root_node], resource)

                for item_id in ids:
                    item = service.find_one(req=None, _id=item_id)

                    if item is not None:
                        print(service.system_update(item_id, updates, item))

    def backwards(self, mongodb_collection, mongodb_database):
        raise NotImplementedError()



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : mugur
# Creation: 2017-11-24 19:54

from copy import deepcopy
from superdesk.commands.data_updates import BaseDataUpdate

from eve.utils import config


class DataUpdate(BaseDataUpdate):
    replace_values_forward = {
        "picture": "media",
        "unorderedlist": "unordered list",
        "orderedlist": "ordered list",
        "anchor": "link",
        "removeFormat": None,
    }
    replace_values_backward = {
        "media": "picture",
        "unordered list": "unorderedlist",
        "ordered list": "orderedlist",
        "link": "anchor",
    }

    resource = "content_types"

    def forwards(self, mongodb_collection, mongodb_database):
        self._process_content_type(mongodb_collection, self.replace_values_forward)

    def backwards(self, mongodb_collection, mongodb_database):
        self._process_content_type(self.replace_values_backward)

    def _process_content_type(self, mongodb_collection, replace_values):
        for content_type in mongodb_collection.find({}):
            if "editor" not in content_type:
                continue
            original_editor = deepcopy(content_type["editor"])
            for field, description in content_type["editor"].items():
                if description and description.get("formatOptions"):
                    for original, new in replace_values.items():
                        if original in description["formatOptions"]:
                            description["formatOptions"].remove(original)
                            if new:
                                description["formatOptions"].append(new)
            if original_editor != content_type["editor"]:
                print("update editor in content type", content_type["label"])
                mongodb_collection.update(
                    {"_id": content_type.get(config.ID_FIELD)}, {"$set": {"editor": content_type["editor"]}}
                )
